 ! Gnuplotter for FORTRAN
!
! Author: Lukasz A. Drozd
! Modified: 8/8/2018
!    
!    This program is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, version 3 of the License.

!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details:
!    <http://www.gnu.org/licenses/>. 
! 
!==========================================================================================================
    
module GnuDataSeriesClass

         implicit none
         
         !====================================================================================================================================================================
         !User defined module settings ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

         logical  :: rungnu=.true. !if true FORTRAN issues system comman in runtime to run gnuplot, if false only output files are created
 
         logical  :: pdfcairo = .true. !if true uses cairo terminal for pdf plots
         logical  :: color = .true. !if false produces monochrome figures in pdf mode
         logical  :: stealthmode=.false. !if true erases all GNU codes and supporting files used to compile figures
         logical  :: pm3d=.false. !if true uses pm3d mode to plot in 3D

         !Available palettes for 3D plots 
         character(len=128), parameter	:: CMY_palette='CMY'
         character(len=128), parameter	:: heat_palette='RGB defined ( 0 "red", 1 "yellow", 2 "white" )'
         character(len=128), parameter	:: rainbow_palette='HSV functions gray,1,1'
         
         character(len=128), parameter	:: saturation_palette='RGB defined ( -1 "blue", 0 "white", 1 "red" )'
         
         character(len=128), parameter	:: hue_palette='RGB defined ( 0 "#3366ff", 1 "#99ffcc", 2 "#339900", 3 "#66ff33", 4 "#996633", 5 "#ff9900", 6 "#ffff33" )'
         
         character(len=128), parameter	:: luminance_palette='HSV defined ( 0 .66666 0.6 0.6, 1 .6666 0.5 1, 2 .33333 0.5 1, 3 .33333 0.7 0.5 )'
          
         !Color bank 1
         character(len=8), parameter    :: colorBank(1:8)=(/"#1B9E77",     "#D95F02",    "#7570B3",    "#E7298A",     "#66A61E",      "#E6AB02",    "#A6761D", "#666666"/)
         !                                                #dark teal    #dark orange   #dark lilac   #dark magenta #dark lime green #dark banana  #dark tan  #dark gray'   
         
         !Color bank 2
         character(len=8), parameter    :: colorBankR(1:4)=(/"#B0171F", "#872657", "#B0171F", "#cd3e32"/) !reds
         character(len=8), parameter    :: colorBankG(1:4)=(/"#006400", "#308014", "#458B00", "#00FF00"/) !greens
         character(len=8), parameter    :: colorBankB(1:4)=(/"#3D59AB", "#104E8B", "#1E90FF", "#36648B"/) !blues
         character(len=8), parameter    :: colorBankM(1:4)=(/"#7b368b", "#63368b", "#7b368b", "#56284d"/) !magenda - violet 
         character(len=8), parameter    :: colorBankGrey(1:4)=(/"#a59f9f", "#6d6a6a", "#4f4b4b", "#3f3c3c" /) !greys

         !Point bank
         character(len=2), parameter :: pointBank(1:11)=(/"17", "4", "8", "25","10","7", "1", "28", "5", "3", "2"/)

         !Default palette for 3D plotting 
         character(len=128)             :: default_palette=luminance_palette
         
         character(len=128)             :: default_colorA=colorBankB(1)
         character(len=128)             :: default_colorB=colorBankG(1)
  
         !end of user settings ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
         !====================================================================================================================================================================
         
         integer, parameter ::  DATA_INTEGER_TYPE   =  1, &
                                DATA_SINGLE_PRECISION_TYPE = 2, &
                                DATA_DOUBLE_PRECISION_TYPE = 3, &
                                DATA_UNDEFINED_TYPE = -1, &
                                DATA_TIME_TYPE=0
         
         real*8, parameter             :: secday=86400.0d0              ! 24:00:00 hours as seconds
         
         type GnuDataSeries 
            character(len=128)                  :: title        = "" 
            character(len=128)                  :: unit         = ""  
            integer                             :: dataType     = DATA_UNDEFINED_TYPE
            integer                             :: offset       = 0
            real*8, allocatable                 :: doubleValues(:)
            real*4, allocatable                 :: singleValues(:)
            integer, allocatable                :: integerValues(:)
            real*8, allocatable                 :: doubleValues2(:,:)
            real*4, allocatable                 :: singleValues2(:,:)
            integer, allocatable                :: integerValues2(:,:)
            real*8, allocatable                 :: timeValues(:)
            logical                             :: userSuppliedStyle=.false.  
            character(len=1)                    :: linestyle='-'
            character(len=8)                    :: linecolor='#3D59AB'
            character(len=1)                    :: linewidth='3'
            integer                             :: size 
            integer                             :: size2 
         end type GnuDataSeries

        interface constructTSeries
            module procedure :: newTSeries
         end interface  
         
         interface constructSeries
            module procedure :: newDPSeries
            module procedure :: newSSeries
            module procedure :: newISeries 
            module procedure :: newDPSeries3D
            module procedure :: newSSeries3D
            module procedure :: newISeries3D
         end interface  
         
         interface epoch 
            module procedure epoch0, epoch1, epoch2, epoch3
	     end interface
         
         private :: newDPSeries, newSSeries, newISeries, newDPSeries3D, newSSeries3D,  newTSeries
         
 contains 
 
     subroutine newDPSeries(DataSeries,title,series, linecolor, linewidth, linestyle, offset, unit)  
     
             implicit none
             type(GnuDataSeries)   :: DataSeries
             
                 character(len=*)  :: title 
                 real*8            :: series(:)
                 character(len=*), optional :: linecolor
                 character(len=*), optional :: linewidth
                 character(len=*), optional :: linestyle 
                 integer, optional  :: offset 
                 character(len=*), optional :: unit

             if( present(unit) ) DataSeries % unit=unit
             
             DataSeries % title        = title
             DataSeries % dataType     = DATA_DOUBLE_PRECISION_TYPE

             DataSeries % size = size(series)
             
             if(allocated(DataSeries % doubleValues)) deallocate(DataSeries % doubleValues)
             allocate(DataSeries % doubleValues(size(series)))
             DataSeries % doubleValues = series
             
             if( present(linecolor) ) DataSeries % linecolor=linecolor
             if( present(linestyle) ) DataSeries % linestyle=linestyle
             if( present(linewidth) ) DataSeries % linewidth=linewidth
             
             if( present(linecolor) ) DataSeries % userSuppliedStyle=.true.
                      
             if(present(offset)) DataSeries % offset = offset
  
     end subroutine newDPSeries
     
     subroutine newSSeries(DataSeries, title, series,  linecolor, linewidth, linestyle, offset, unit)  
     
             implicit none
             type(GnuDataSeries) :: DataSeries
                 character(len=*)       :: title 
                 real*4  :: series(:)
                 character(len=*), optional  ::  linecolor, linewidth, linestyle 
                 integer, optional :: offset
                 character(len=*), optional :: unit

             if( present(unit) ) DataSeries % unit=unit

             DataSeries % title        = title
             DataSeries % dataType     = DATA_SINGLE_PRECISION_TYPE
         
             if(present(offset)) DataSeries % offset= offset 
         
             if(allocated(DataSeries % singleValues)) deallocate(DataSeries % singleValues)
             allocate(DataSeries % singleValues(size(series)))
             DataSeries % singleValues = series
                      
             if( present(linecolor) ) DataSeries % linecolor=linecolor
             if( present(linecolor) ) DataSeries % linestyle=linestyle
             if( present(linewidth) ) DataSeries % linewidth=linewidth
             
             if( present(linecolor) ) DataSeries % userSuppliedStyle=.true.
             
            DataSeries % size = size(series)
                          
     end subroutine newSSeries
     
     subroutine newISeries(DataSeries, title, series,  linecolor, linewidth, linestyle, offset, unit)  
     
             implicit none
             type(GnuDataSeries) :: DataSeries
                 character(len=*)       :: title 
                 integer  :: series(:)
                 character(len=*), optional :: linecolor
                 character(len=*), optional :: linewidth
                 character(len=*), optional :: linestyle 
                 integer, optional  :: offset 
                 character(len=*), optional :: unit

             if( present(unit) ) DataSeries % unit=unit

             DataSeries % title        = title
             DataSeries % dataType     = DATA_INTEGER_TYPE
         
             if(present(offset)) DataSeries % offset= offset 
             
             if(allocated(DataSeries % integerValues)) deallocate(DataSeries % integerValues)
                
             allocate(DataSeries % integerValues(size(series)))
             DataSeries % integerValues = series
                      
             if( present(linecolor) ) DataSeries % linecolor=linecolor
             if( present(linestyle) ) DataSeries % linestyle=linestyle
             if( present(linewidth) ) DataSeries % linewidth=linewidth
             
             if( present(linecolor) ) DataSeries % userSuppliedStyle=.true.
             
             DataSeries % size = size(series)
             
     end subroutine newISeries 

      subroutine newDPSeries3D(DataSeries,title,series, linecolor, linewidth, linestyle, offset )  
     
             implicit none
             type(GnuDataSeries)   :: DataSeries
                 character(len=*)  :: title 
                 real*8            :: series(:,:)
                 character(len=*), optional :: linecolor
                 character(len=*), optional :: linewidth
                 character(len=*), optional :: linestyle 
                 integer, optional  :: offset 

             DataSeries % title        = title
             DataSeries % dataType     = DATA_DOUBLE_PRECISION_TYPE

             if(allocated(DataSeries % doubleValues2)) deallocate(DataSeries % doubleValues2)
             allocate(DataSeries % doubleValues2( size(series,1),size(series,2) ))
             DataSeries % doubleValues2 = series
             
             if( present(linecolor) ) DataSeries % linecolor=linecolor
             if( present(linestyle) ) DataSeries % linestyle=linestyle
             if( present(linewidth) ) DataSeries % linewidth=linewidth
             
             if( present(linecolor) ) DataSeries % userSuppliedStyle=.true.
                      
             if(present(offset)) DataSeries % offset= offset
             
             DataSeries % size = size(series,1)
             DataSeries % size2 = size(series,2)
         
     end subroutine newDPSeries3D
     
     subroutine newSSeries3D(DataSeries, title, series,  linecolor, linewidth, linestyle, offset )  
     
             implicit none
             type(GnuDataSeries) :: DataSeries
                 character(len=*)       :: title 
                 real*4  :: series(:,:)
                 character(len=*), optional :: linecolor
                 character(len=*), optional :: linewidth
                 character(len=*), optional :: linestyle 
                 integer, optional  :: offset 

             DataSeries % title        = title
             DataSeries % dataType     = DATA_SINGLE_PRECISION_TYPE
         
             if(present(offset)) DataSeries % offset= offset 
         
             if(allocated(DataSeries % singleValues2)) deallocate(DataSeries % singleValues2)
             allocate(DataSeries % singleValues2(size(series,1),size(series,2)))
             DataSeries % singleValues2 = series
                      
             if( present(linecolor) ) DataSeries % linecolor=linecolor
             if( present(linecolor) ) DataSeries % linestyle=linestyle
             if( present(linewidth) ) DataSeries % linewidth=linewidth
             
             if( present(linecolor) ) DataSeries % userSuppliedStyle=.true.
             
            DataSeries % size = size(series,1)
            DataSeries % size2 = size(series,2)
                          
     end subroutine newSSeries3D

     subroutine newISeries3D(DataSeries, title, series,  linecolor, linewidth, linestyle, offset)  
     
             implicit none
             type(GnuDataSeries)    :: DataSeries
                 character(len=*)   :: title 
                 integer            :: series(:,:) 
                 character(len=*), optional :: linecolor
                 character(len=*), optional :: linewidth
                 character(len=*), optional :: linestyle 
                 integer, optional  :: offset 
 
             DataSeries % title        = title
             DataSeries % dataType     = DATA_INTEGER_TYPE
 
             if(present(offset)) DataSeries % offset= offset 
         
             if(allocated(DataSeries % integerValues2)) deallocate(DataSeries % integerValues2)
             allocate(DataSeries % integerValues2(size(series,1),size(series,2)))
             DataSeries % integerValues2 = series
                      
             if( present(linecolor) ) DataSeries % linecolor=linecolor
             if( present(linestyle) ) DataSeries % linestyle=linestyle
             if( present(linewidth) ) DataSeries % linewidth=linewidth
             
             if( present(linecolor) ) DataSeries % userSuppliedStyle=.true.
             
             DataSeries % size = size(series,1)
             DataSeries % size2 = size(series,2)
             
     end subroutine newISeries3D 
 
     subroutine newTSeries(DataSeries, title, series,  linecolor, linewidth, linestyle, offset, unit)  
     
             implicit none
             type(GnuDataSeries)    :: DataSeries
                 character(len=*)   :: title 
                 integer            :: series(:,:) 
                 character(len=*), optional :: linecolor
                 character(len=*), optional :: linewidth
                 character(len=*), optional :: linestyle 
                 integer, optional  :: offset 
                 character(len=*), optional :: unit

             if( present(unit) ) DataSeries % unit=unit
 
             DataSeries % title        = title
             DataSeries % dataType     = DATA_TIME_TYPE
 
             if(allocated(DataSeries % timeValues)) deallocate(DataSeries % timeValues)
             allocate(DataSeries % timeValues(size(series, 1)))
              
             call getEpoch(series, DataSeries%timeValues)
                      
             if( present(linecolor) ) DataSeries % linecolor=linecolor
             if( present(linestyle) ) DataSeries % linestyle=linestyle
             if( present(linewidth) ) DataSeries % linewidth=linewidth
             
             if( present(linecolor) ) DataSeries % userSuppliedStyle=.true.
             
             DataSeries % size = size(series,1)
             DataSeries % size2 = size(series,2)
             
      end subroutine newTSeries 
 
      subroutine getEpoch(input, dataout) 
 
        integer,intent(in) :: input(:,:)
        real*8,intent(inout) :: dataout(:)
        real*8 :: unixtime
    
        integer :: i, ierr
        
        integer,allocatable :: datain(:,:)
 
        allocate(datain(1:size(input,1),1:7))
        datain(1:size(input,1),1:3) =1
        datain(1:size(input,1),4:7) =0
        datain(1:size(input,1), 1:size(input,2))=input
        
        do i=1,size(datain,1)
            
            call d2u(datain(i,1:7),dataout(i),ierr)     
            
            if(ierr .ne. 0 ) then 
                
                print*, 'Warning: error in conversion to unix time!' 
            
                select case (ierr)
                    case (-1)
                    print*,'            => invalid year!'     
                    case (1)
                    print*,'            => Julian Day negative!'      
                end select 
            end if 
            
        end do 
     
        deallocate(datain)
     
     end subroutine getEpoch

     subroutine d2j(dat,julian,ierr)
     !Converts date array to Julian Day  
 
        integer,intent(in)         :: dat(1:7)   ! array like returned by DATE_AND_TIME(3f)
        real*8,intent(out)         :: julian   ! Julian Day (non-negative, but may be non-integer)
        integer,intent(out)        :: ierr     ! Error return, 0 for successful execution,-1=invalid year,1=invalid date (29th Feb, non leap-year)
               integer                 :: year, month, day, utc, hour, minute
               real*8                  :: second
               integer                 :: A, Y, M, JDN
               year   = dat(1)                        ! Year
               month  = dat(2)                        ! Month
               day    = dat(3)                        ! Day        
               utc    = dat(4)*60                     ! Delta from UTC, convert from minutes to seconds
               hour   = dat(5)                        ! Hour
               minute = dat(6)                        ! Minute
               second = dat(7)-utc     ! Second  
               julian = -HUGE(99999)                  
               if(year==0 .or. year .lt. -4713) then
                  ierr=-1
                  return
               endif
             ! Compute first the number of years (Y) and months (M) since March 1st -4800 (March 1, 4801 BC)
               A=(14-month)/12 ! A will be 1 for January or Febuary, and 0 for other months, with integer truncation
               Y=year+4800-A
               M=month+12*A-3  ! M will be 0 for March and 11 for Febuary
            !  All years in the BC era must be converted to astronomical years, so that 1BC is year 0, 2 BC is year "-1", etc.
            !  Convert to a negative number, then increment towards zero
            !  Staring from a Gregorian calendar date
               JDN=day + (153*M+2)/5 + 365*Y + Y/4 - Y/100 + Y/400 - 32045  !  with integer truncation
            !  Finding the Julian date given the JDN (Julian day number) and time of day
               julian=JDN + dble(hour-12)/24.0d0 + dble(minute)/1440.0d0 + second/86400.0d0
 
               if(julian.lt.0.d0) then                  ! Julian Day must be non-negative
                  ierr=1
               else
                  ierr=0
               endif
               
     end subroutine d2j
 
     subroutine d2u(dat,unixtime,ierr)
            !Converts date array to Unix Time (UT starts at 0000 on 1 Jan. 1970)
            integer,intent(in)         :: dat(1:7)                  ! date time array similar to that returned by DATE_AND_TIME
            real*8,intent(out)         :: unixtime                ! Unix time (seconds)
            integer,intent(out)        :: ierr                    ! return 0 on successful, otherwise 1
               real*8                  :: julian
               real*8,save             :: julian_at_epoch
               logical,save            :: first=.true.
 
            if(first) then                                        ! Compute zero of Unix Time in Julian days and save
               call d2j([1970,1,1,0,0,0,0],julian_at_epoch,ierr)
               if(ierr.ne.0) return                               ! Error
               first=.false.
            endif
 
            call d2j(dat,julian,ierr)
            if(ierr.ne.0) return                               ! Error
            unixtime=(julian-julian_at_epoch)*secday
     end subroutine d2u
 
     function epoch0(year,month,day,utc,hour,minute,second) ![must upconvert to double to avoid binding integer 32 limit]
            !Converts date  to Unix Time (UT starts at 0000 on 1 Jan. 1970)
            integer,intent(in)         :: year, month, day, utc, hour, minute, second
 
            real*8                  :: julian
            real*8,save             :: julian_at_epoch
            logical,save            :: first=.true.
               
            integer:: dat(1:7), ierr                     ! date time array similar to that returned by DATE_AND_TIME
            real*8 :: epoch0
             
               dat(1)= year                        ! Year
               dat(2)= month                       ! Month
               dat(3)= day                         ! Day        
               dat(4)= utc                         ! Delta from UTCin minutes
               dat(5)= hour                        ! Hour
               dat(6)= minute                      ! Minute
               dat(7)= second                      ! Second  
               
            if(first) then                                        ! Compute zero of Unix Time in Julian days and save
               call d2j([1970,1,1,0,0,0,0],julian_at_epoch,ierr)
               if(ierr.ne.0) return                               ! Error
               first=.false.
            endif
 
            call d2j(dat,julian,ierr)
            if(ierr.ne.0) return                               ! Error
            epoch0=(julian-julian_at_epoch)*secday
            
    end function epoch0 
          
    function epoch1 (yyyy, mm,dd) 
    !converts calendar date to epoch time for time series plotting

        integer,intent(in) :: yyyy , mm, dd
        real*8 :: epoch1

        !output : date ->  julian date -> epoch [must upconvert to double to avoid binding integer 32 limit]

        epoch1 = dble( dd-32075.0d0+1461.0d0*(yyyy+4800.0d0+(mm-14.0d0)/12)/4 + &
                 367*(mm-2-((mm-14)/12)*12)/12- &
                 3*((yyyy + 4900.0d0 + (mm - 14)/12)/100)/4 - 2440588  )*secday
 
    end function epoch1

    function epoch2 ( yyyy,mm)
    !converts calendar date to epoch time for time series plotting

        integer,intent(in) :: yyyy , mm
        integer ::  dd
        real*8 :: epoch2

        dd=1

        !output : date ->  julian date -> epoch [must upconvert to double to avoid binding integer 32 limit]

        epoch2 = dble( dd-32075+1461*(yyyy+4800+(mm-14)/12)/4 + &
                 367*(mm-2-((mm-14)/12)*12)/12- &
                 3*((yyyy + 4900 + (mm - 14)/12)/100)/4 - 2440588 )*secday
 
    end function epoch2

 
    function epoch3 (yyyy) 
    !converts calendar date to epoch time for time series plotting

        integer,intent(in) :: yyyy 

        real*8 :: epoch3
        integer :: mm, dd

        mm=1; dd=1

        !output : date ->  julian date -> epoch [must upconvert to double to avoid binding integer 32 limit]

        epoch3 = dble( dd-32075+1461*(yyyy+4800+(mm-14)/12)/4 + &
                        367*(mm-2-((mm-14)/12)*12)/12- &
                            3*((yyyy + 4900 + (mm - 14)/12)/100)/4 - 2440588 )*secday
 
    end function epoch3
     
    function getNumberOfDataPoints(DataSeries) RESULT(n)
             implicit none
             type(GnuDataSeries)      :: DataSeries
                integer                  :: n
         
             select case ( DataSeries % dataType )
                case( DATA_SINGLE_PRECISION_TYPE ) 
                   n = SIZE(DataSeries % singleValues) 
                case( DATA_DOUBLE_PRECISION_TYPE ) 
                   n = SIZE(DataSeries % doubleValues) 
                case( DATA_TIME_TYPE ) 
                   n = SIZE(DataSeries % timeValues) 
                case( DATA_INTEGER_TYPE ) 
                   n = SIZE(DataSeries % integerValues) 
                case default
                   n = 0
                end select
            
    end function getNumberOfDataPoints

end module GnuDataSeriesClass
 
!==============================================================================================================================================  
    
module GnuFigureMakerClass 

    use GnuDataSeriesClass

    implicit none
    !default parameters which control linewidth, colors and terminal

        type:: LabelType
            character(len=100), private                    :: label= ""
            real*8, private                                :: xpos=0.0d0
            real*8, private                                :: ypos=0.0d0
            real*8, private                                :: zpos=0.0d0
            integer, private                               :: rotation=0
            character(len=8), private                      :: color= "#000000"
        end type LabelType
        
        type:: ArrowType
            real*8, private                                :: from_xpos=0.0d0
            real*8, private                                :: from_ypos=0.0d0
            real*8, private                                :: from_zpos=0.0d0
            real*8, private                                :: to_xpos=0.01d0
            real*8, private                                :: to_ypos=0.01d0
            real*8, private                                :: to_zpos=0.01d0
            character(len=8), private                      :: color= "#000000"
        end type ArrowType
        
        type:: CommandType
            character(len=128), private                    :: command= ""
        end type CommandType
                   
        type :: GnuFigureMaker 
            integer, private                                       :: dimension_= 2
            character(len=80), private                             :: title_= ""
            character(len=30), private                             :: xlabel_= "x"
            character(len=30), private                             :: ylabel_= "y"
            character(len=30), private                             :: zlabel_= "z"
            character(len=30), private                             :: datefmt_= "%Y"
            
            integer, private                                       :: xaxismode_=1
            logical, private                                       :: grids_=.true.
            logical, private                                       :: legend_=.true.

            real*8, private                                        :: uppermarginy_=0.
            real*8, private                                        :: lowermarginy_=0.
            real*8, private                                        :: uppermarginz_=0.
            real*8, private                                        :: lowermarginz_=0. 
            real*8, private                                        :: uppermarginx_=0.
            real*8, private                                        :: lowermarginx_=0.
            
            type(LabelType), dimension(:), allocatable, private    :: labels_ 
            integer, private                                       :: numlabels_=0
            type(ArrowType), dimension(:), allocatable, private    :: arrows_ 
            integer, private                                       :: numarrows_=0
            type(CommandType), dimension(:), allocatable, private  :: commands_ 
            integer, private                                       :: numcommands_=0
    
        contains 

                   procedure :: setTitle
                   procedure :: clearTitle
                   procedure :: setDimension
                   procedure :: setAxisLabels
                   procedure :: clearAxisLabels
                   procedure :: setTimeFormat
                   procedure :: clearTimeFormat 
                   procedure :: setMargins
                   procedure :: clearMargins
                   procedure :: setGrids
                   procedure :: clearGrids
                   procedure :: setLegend
                   procedure :: clearLegend
                   procedure :: addLabel
                   procedure :: clearLabels
                   procedure :: addArrow
                   procedure :: clearArrows
                   procedure :: addCommand
                   procedure :: clearCommands
                   procedure :: gnuplot2D 
                   procedure :: gnusubplot2D 
                   procedure :: gnuplot3D_auto
                   procedure :: gnuplot3D_auto_2
                   procedure :: gnuplot3D
                   procedure :: gnuplot3D_2  
                   procedure :: gnuhistogram 
 
        end type GnuFigureMaker
 
contains
 
    subroutine clearAxisLabels(self )  
         implicit none
         class(GnuFigureMaker)                :: self
         
         self % xlabel_ = "x"
         self % ylabel_ = "y" 
         self % zlabel_ = "z" !restore default setting
 
    end subroutine clearAxisLabels
    
    subroutine setAxisLabels(self, xaxis, yaxis, zaxis)  
         implicit none
         class(GnuFigureMaker)                :: self
             character(len=*)                 :: xaxis, yaxis
             character(len=*), optional       :: zaxis
         
         self % xlabel_ = xaxis
         self % ylabel_ = yaxis 
         
         if ( present(zaxis)) then 
            self % zlabel_ = zaxis
         else 
            self % zlabel_ = "z" !restore default setting
         end if

    end subroutine setAxisLabels
    
    subroutine setTitle(self, title)  
         implicit none
         class(GnuFigureMaker)       :: self
            character(len=*)            :: title
         
         self % title_ = title 
        
    end subroutine setTitle

    subroutine clearTitle(self)  
         implicit none
         class(GnuFigureMaker)       :: self 
         
         self % title_ = ""
        
    end subroutine clearTitle

    subroutine clearMargins(self)  
         implicit none
         class(GnuFigureMaker)       :: self 
         
         self % lowermarginy_ = 0.
         self % uppermarginy_ = 0.
 
         self % lowermarginz_ = 0.
         self % uppermarginz_ = 0.
        
    end subroutine clearMargins
 
    subroutine setMargins(self,  downy, upy,  downz, upz,  downx, upx)  
         implicit none
         class(GnuFigureMaker)       :: self
            real*8                        :: downy, upy
            real*8, optional              :: downz, upz
              real*8, optional              :: downx, upx
         
         self % lowermarginy_ = downy
         self % uppermarginy_ = upy

         if ( present(downz) ) then 
            self % lowermarginz_ = downz
            self % uppermarginz_ = upz
         end if
         
         if ( present(downz) ) then 
            self % lowermarginx_ = downx
            self % uppermarginx_ = upx
         end if
        
    end subroutine setMargins

    subroutine clearLegend(self)  
         implicit none
         class(GnuFigureMaker)       :: self

         self % legend_ = .false.  
        
    end subroutine clearLegend
        
    subroutine setLegend(self)  
         implicit none
         class(GnuFigureMaker)       :: self

         self % legend_ = .true.  
        
    end subroutine setLegend
    
    subroutine setGrids(self)  
         implicit none
         class(GnuFigureMaker)       :: self

         self % grids_ = .true.  
        
   end subroutine setGrids

   subroutine clearGrids(self)  
         implicit none
         class(GnuFigureMaker)       :: self

         self % grids_ = .false.  
        
    end subroutine clearGrids
 
    subroutine setDimension(self, dim)  
         implicit none
         class(GnuFigureMaker)       :: self
         integer                     :: dim
         
         self % dimension_ = dim
         if ( dim >3 .or. dim<2 ) then 
            print*, 'Illegal value of dimension in setDimension constructor!'
         end if
        
    end subroutine setDimension

    subroutine addArrow(self, from_xpos, from_ypos,  to_xpos, to_ypos, color)  
         implicit none
         class(GnuFigureMaker)  :: self 
 
            real*8  :: from_xpos, from_ypos
            real*8  :: to_xpos, to_ypos 
            character(len=*)  , optional :: color
 
            integer :: n 
         
            type(ArrowType), dimension(:), allocatable :: temparrows
         
         self % numarrows_ = self % numarrows_+1
         
         if( allocated(self % arrows_) ) then 
             
            n= size(self % arrows_)
            
            allocate(temparrows(1:n))
            temparrows(1:n)=self % arrows_(1:n)
            
            deallocate (self % arrows_)
        
            allocate (self % arrows_(1:n+1))
            self % arrows_(1:n)=temparrows(1:n)

            self % arrows_(n+1) % from_xpos = from_xpos
            self % arrows_(n+1) % from_ypos = from_ypos
            !self % arrows_(n+1) % from_zpos = from_zpos
            self % arrows_(n+1) % to_xpos = to_xpos
            self % arrows_(n+1) % to_ypos = to_ypos
            !self % arrows_(n+1) % to_zpos = to_zpos
            
            if(present(color) ) self % arrows_(n+1) % color = color
                    
            deallocate(temparrows)

        else
        
            allocate(self % arrows_(1))
 
            self % arrows_(1) % from_xpos = from_xpos
            self % arrows_(1) % from_ypos = from_ypos
            !self % arrows_(1) % from_zpos = from_zpos
            self % arrows_(1) % to_xpos = to_xpos
            self % arrows_(1) % to_ypos = to_ypos
            !self % arrows_(1) % to_zpos = to_zpos
            
            if(present(color) ) self % arrows_(1) % color = color
        
        end if 
        
    end subroutine addArrow

    subroutine addArrow3D(self, from_xpos, from_ypos, from_zpos, to_xpos, to_ypos, to_zpos, color)  
         implicit none
         class(GnuFigureMaker)  :: self 
 
            real*8  :: from_xpos, from_ypos, from_zpos
            real*8  :: to_xpos, to_ypos, to_zpos
            character(len=*)  , optional :: color
 
            integer :: n 
         
            type(ArrowType), dimension(:), allocatable :: temparrows
         
         self % numarrows_ = self % numarrows_+1
         
         if( allocated(self % arrows_) ) then 
             
            n= size(self % arrows_)
            
            allocate(temparrows(1:n))
            temparrows(1:n)=self % arrows_(1:n)
            
            deallocate (self % arrows_)
        
            allocate (self % arrows_(1:n+1))
            self % arrows_(1:n)=temparrows(1:n)

            self % arrows_(n+1) % from_xpos = from_xpos
            self % arrows_(n+1) % from_ypos = from_ypos
            self % arrows_(n+1) % from_zpos = from_zpos
            self % arrows_(n+1) % to_xpos = to_xpos
            self % arrows_(n+1) % to_ypos = to_ypos
            self % arrows_(n+1) % to_zpos = to_zpos
            
            if(present(color) ) self % arrows_(n+1) % color = color

            deallocate(temparrows)
            
        else
        
            allocate(self % arrows_(1))
 
            self % arrows_(1) % from_xpos = from_xpos
            self % arrows_(1) % from_ypos = from_ypos
            self % arrows_(1) % from_zpos = from_zpos
            self % arrows_(1) % to_xpos = to_xpos
            self % arrows_(1) % to_ypos = to_ypos
            self % arrows_(1) % to_zpos = to_zpos
            
            if(present(color) ) self % arrows_(1) % color = color
        
        end if 
        
    end subroutine addArrow3D
    
    subroutine clearArrows(self )  
    
         implicit none
         
         class(GnuFigureMaker)  :: self 
         
         self % numarrows_ = 0
         
         deallocate (self % arrows_)
         
    end subroutine clearArrows
    
    subroutine addCommand(self, command  )  
    
         implicit none
         
         class(GnuFigureMaker)                      :: self 
         
            character(len=*)                        :: command 
            integer :: n 
         
            type(CommandType), dimension(:), allocatable :: tempcommands
         
         self % numcommands_ = self % numcommands_+1
         
         if( allocated(self % commands_) ) then 
             
            n= size(self % commands_)
            allocate(tempcommands(1:n))
            tempcommands(1:n)=self % commands_(1:n)
            
            deallocate (self % commands_)
        
            allocate (self % commands_(1:n+1))
            self % commands_(1:n)=tempcommands(1:n)
            self % commands_(n+1) % command =command
  
            deallocate(tempcommands)
            
        else
        
             allocate(self % commands_(1))
             self % commands_(1) % command    = command 
        
        end if
                
    end subroutine addCommand
    
    subroutine clearCommands(self )  
    
         implicit none
         class(GnuFigureMaker) :: self 
         
         self % numcommands_ = 0
         
         deallocate (self % commands_)
         
    end subroutine clearCommands

    subroutine addLabel3D(self, label, xpos, ypos, rotation, color)  
    
         implicit none
         
         class(GnuFigureMaker)                      :: self 
         
            character(len=*)                        :: label
            real*8                                  :: xpos, ypos, zpos
            integer                                 :: rotation
            character(len=*), optional              :: color
            integer :: n 
         
            type(LabelType), dimension(:), allocatable :: templabels
         
         self % numlabels_ = self % numlabels_+1
         
         if( allocated(self % labels_) ) then 
             
            n= size(self % labels_)
            allocate(templabels(1:n))
            templabels(1:n)=self % labels_(1:n)
            
            deallocate (self % labels_)
        
            allocate (self % labels_(1:n+1))
            self % labels_(1:n)=templabels(1:n)
            self % labels_(n+1) % label =label
            self % labels_(n+1) % xpos = xpos
            self % labels_(n+1) % ypos = ypos
            self % labels_(n+1) % zpos = zpos
            self % labels_(n+1) % rotation = rotation
            if(present(color) ) self % labels_(n+1) % color = color
 
            deallocate(templabels)
             
        else
        
             allocate(self % labels_(1))
             self % labels_(1) % label    = label
             self % labels_(1) % xpos     = xpos
             self % labels_(1) % ypos     = ypos
            self % labels_(1) % ypos     = zpos
             self % labels_(1) % rotation = rotation
             if(present(color) ) self % labels_(1) % color = color
        
        end if 

    end subroutine addLabel3D
 
    subroutine addLabel(self, label, xpos, ypos, rotation, color)  
    
         implicit none
         
         class(GnuFigureMaker)                      :: self 
         
            character(len=*)                        :: label
            real*8                                  :: xpos, ypos
            integer                                 :: rotation
            character(len=*), optional              :: color
            integer :: n 
         
            type(LabelType), dimension(:), allocatable :: templabels
         
         self % numlabels_ = self % numlabels_+1
         
         if( allocated(self % labels_) ) then 
             
            n= size(self % labels_)
            allocate(templabels(1:n))
            templabels(1:n)=self % labels_(1:n)
            
            deallocate (self % labels_)
        
            allocate (self % labels_(1:n+1))
            self % labels_(1:n)=templabels(1:n)
            self % labels_(n+1) % label =label
            self % labels_(n+1) % xpos = xpos
            self % labels_(n+1) % ypos = ypos
            self % labels_(n+1) % rotation = rotation
            if(present(color) ) self % labels_(n+1) % color = color
            
            deallocate(templabels)

             
        else
        
             allocate(self % labels_(1))
             self % labels_(1) % label    = label
             self % labels_(1) % xpos     = xpos
             self % labels_(1) % ypos     = ypos
             self % labels_(1) % rotation = rotation
             if(present(color) ) self % labels_(1) % color = color
        
        end if  

    end subroutine addLabel
    
    subroutine clearLabels(self )  
    
         implicit none
         class(GnuFigureMaker) :: self 
         
         self % numlabels_ = 0
         
         deallocate (self % labels_)
         
    end subroutine clearLabels
    
    subroutine setTimeFormat(self, fmt)  
    
         implicit none
         class(GnuFigureMaker) :: self 
 
            character(len=*), optional  :: fmt
         
         self % xaxismode_ = 2
         if ( present(fmt)) then 
            self % datefmt_ = fmt
         else 
            self % datefmt_ = "%Y" !restore default setting
         end if
        
    end subroutine setTimeFormat
     
    subroutine clearTimeFormat(self)  
    
         implicit none
         class(GnuFigureMaker) :: self 
  
 
         self % datefmt_ = "%Y" !restore default setting
  
    end subroutine clearTimeFormat
    
    subroutine gnuplot2D(self, DataX, DataY, terminal, label, stealthin ) 

	implicit none
    integer, parameter		    :: Nc=32, Nclong=128
    class(GnuFigureMaker)       :: self
    type(GnuDataSeries)         :: DataX 
    type(GnuDataSeries)         :: DataY(:) 
    character(len=*), optional  :: terminal
    character(len=*), optional  :: label
    logical, optional           :: stealthin
    character(len=Nclong)       :: my_text_tmp1, my_text_tmp2, my_text_tmp3, my_text_tmp4
	character(len=Nclong)	    :: output_file_name, data_file_name, command_file_name, tex_file_name

    character(len=Nc), dimension(:), allocatable   :: xdat 
    character(len=Nc), dimension(:), allocatable   :: my_line_style 
    character(len=Nc), dimension(:,:), allocatable :: ydat
    integer, dimension(:), allocatable :: arrayn

    real*8 :: maxy, miny
    
    integer   :: j, i, ios, file_unit, nmax, nx, ny, terminalme 
    logical   :: stealth 
    
    if( self%dimension_ .ne. 2 ) stop 'Error in gnuplotClass: dimension specification inconsistent!' 
    
    if( present(stealthin) ) then 
        stealth=stealthin
    else 
        stealth=stealthmode
    end if 

    !check if output file present 
    if ( present(terminal) ) then 
        if( terminal == 'screen' ) then 
            terminalme=0    
        else 
            terminalme=1
            if( .not. present(label)) then 
           		    print *,'Fatal error in gnuplotter: Must specify output goes to files!'
		            stop  
            end if
            
            output_file_name='gnu-fig-'//trim(label)//'.'//trim(terminal)
                        
        end if 
    else
        terminalme=0
    end if
    
 !determine presence of input identifier 
	if (present(label)) then
		data_file_name='gnu-dat-'//trim(label)//'.txt'
		command_file_name='gnu-com-'//trim(label)//'.txt'		
        tex_file_name='gnu-cairo-'//trim(label)//'.tex'
	else
		data_file_name='temp-data-'//trim(timestamp())//'.txt'
		command_file_name='temp-command-'//trim(timestamp())//'.txt'
        tex_file_name='gnu-cairo-'//trim(timestamp())//'.tex"'
        stealth=.true.
    end if
 
    ny=size(DataY) ; nmax=0
    
    allocate( arrayn(1:ny), my_line_style(1:ny)  )  
    
    do j=1,ny
        arrayn(j)   = DataY(j)%size+DataY(j)%offset
    end do 
     
    nmax=maxval(arrayn(1:ny))
 
        nx=DataX%size
        
        if(nx .ne. nmax) then 
            print*, 'Fatal error in gnuplotter: array size mismatch!'
            return
        end if
 
    !create char output arrays
    
    allocate(xdat(1:nmax)) 
    allocate(ydat(1:nmax,1:ny)) 

    !open file
 	
	call get_unit(file_unit)	
	if (file_unit==0) then 
		print *,'Fatal error in gnuplotter: Could not get a free FORTRAN unit!'
		return
	end if
	open (unit=file_unit, file=data_file_name, status='replace', iostat=ios)	
	if (ios/=0) then 
		print *,'Fatal error in gnuplotter: Could not open data file!'
		return
    end if
    
    maxy=0.; miny=0.
 
    do j=1,ny
            
            select case( DataY(j) % dataType )
            
                case( DATA_SINGLE_PRECISION_TYPE )
                
                    do i=1,nmax
                        
                            if(i<=arrayn(j) .and. i>DataY(j)%offset ) then 
                                write (ydat(i,j),'(e32.16)') DataY(j)%singleValues(i+DataY(j)%offset) 
                                maxy=max(maxy,dble(DataY(j)%singleValues(i+DataY(j)%offset)  ))
                                miny=min(miny,dble(DataY(j)%singleValues(i+DataY(j)%offset)  ))
                            else 
                                write (ydat(i,j),'(a)') ''
                            end if
                      end do 
        
                case( DATA_DOUBLE_PRECISION_TYPE )
                    
                     do i=1,nmax
                         
                            if(i<=arrayn(j) .and. i>DataY(j)%offset) then 
                                write (ydat(i,j),'(e32.16)') DataY(j)%doubleValues(i+DataY(j)%offset) 
                                maxy=max(maxy, DataY(j)%doubleValues(i+DataY(j)%offset)  )
                                miny=min(miny, DataY(j)%doubleValues(i+DataY(j)%offset)  )
                            else 
                                write (ydat(i,j),'(a)') ''
                            end if 
                            
                      end do 
            
                case( DATA_INTEGER_TYPE )  
                        
                      do i=1,nmax
                            
                        if(i<=arrayn(j) .and. i>DataY(j)%offset) then 
                                write (ydat(i,j),'(i8)') DataY(j)%integerValues(i+DataY(j)%offset) 
                                maxy=max(maxy,dble(DataY(j)%integerValues(i+DataY(j)%offset)   ))
                                miny=min(miny,dble(DataY(j)%integerValues(i+DataY(j)%offset)   ))
                         else 
                                write (ydat(i,j),'(a)') ''
                         end if   
                        
                      end do 
 
                 case default
                            print*, 'Error in gnuplotClass: Input data type illegal!'
                 end select   
    end do 
    
    maxy=maxy+self % uppermarginy_
    miny=miny-self % lowermarginy_
                    
    do i=1,nmax

        if( DataX % dataType==DATA_TIME_TYPE) write (xdat(i),'(e32.16)')   DataX%timeValues(i) !time axis
        if( DataX % dataType==DATA_INTEGER_TYPE) write (xdat(i),'(i8)') DataX%integerValues(i)
        if( DataX % dataType==DATA_SINGLE_PRECISION_TYPE) write (xdat(i),'(e32.16)') DataX%singleValues(i)
        if( DataX % dataType==DATA_DOUBLE_PRECISION_TYPE) write (xdat(i),'(e32.16)') DataX%doubleValues(i)

    end do
    
    !write data to data_file - the gnuplot will read this data later 
 
	write (file_unit,'(a)', advance='no')  '# '//trim(DataX%title)//',   ' 
    do j=1,ny-1
            write (file_unit,'(a)', advance='no')   trim(DataY(j)%title)//',   ' 
    end do 
    write (file_unit,'(a)', advance='yes')   trim(DataY(j)%title) 
 
	do i=1,nmax
		write (file_unit,'(a)',advance='no') trim(xdat(i))//', ' 
        do j=1,ny-1
            write (file_unit,'(a)', advance='no') trim(ydat(i,j))//', '
        end do 
        write (file_unit,'(a)',advance='yes') trim(ydat(i,ny)) 
	end do
	write (file_unit,'(a)')
 
	close (unit=file_unit) !data file has been prepared
    
    !prepare command file
 
	call get_unit(file_unit)
	if (file_unit==0) then 
		print *,'Fatal error in gnuplotter: Could not get a free FORTRAN unit!'
		return
	end if
	open (unit=file_unit, file=command_file_name, status='replace',	iostat=ios)
	if (ios/=0) then 
		print *,'Fatal error in gnuplotter: Could not open the terminal command file!'
		return
    end if	
    
    !write gnu commands to command file - gnuplot will read this later 
    
    !read optional setttings
 
    do j=1,ny
	    if ( DataY(j)%userSuppliedStyle==.true. )  then
            
                if ((DataY(j)%linestyle(1:1)=='.')) then
                    my_line_style(j)='points pointtype ' // pointBank(min(11,j)) 
                elseif ((DataY(j)%linestyle(1:1)==':')) then    
		            my_line_style(j)='linespoints pointtype ' // pointBank(min(11,j)) 
                else
		            my_line_style(j)='lines'
                end if  
         else 
               my_line_style(j)='lines'
         end if 
    end do 
  
    write ( file_unit, '(a)' ) 'reset'
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Datafile seperator'
    write ( file_unit, '(a)' ) 'set datafile separator ","'

    if( pdfcairo==.true. .and. terminalme==1 ) then 
            write ( file_unit, '(a)' ) ''
            write ( file_unit, '(a)' ) '# Terminal'
            write ( file_unit, '(a)' ) 'set terminal cairolatex standalone pdf size 16cm,10.5cm dashed transparent \'
            if(color==.false.) write ( file_unit, '(a)' ) ' monochrome header ''\newcommand{\hl}[1]{\setlength{\fboxsep}{0.75pt}\colorbox{white}{#1}}'' '
            if(color==.true.) write ( file_unit, '(a)' ) 'color header ''\newcommand{\hl}[1]{\setlength{\fboxsep}{0.75pt}\colorbox{white}{#1}}'''
            
            !write ( file_unit, '(a)' ) 'set terminal epslatex size 9cm,7cm color colortext standalone header \'
            !write ( file_unit, '(a)' ) ' "\\newcommand{\\ft}[0]{\\footnotesize}"'
            
            write ( file_unit, '(a)' ) 'set output '//'"'//trim(tex_file_name)//'"'

    elseif( pdfcairo==.false. .and. terminalme==1 ) then
        
        write ( file_unit, '(a)' ) 'set terminal pdf' !font "Verdana,8"
        
    else  
            write ( file_unit, '(a)' ) 'set terminal  wxt enhanced' !font "Verdana,8"
    end if
    
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Legend'
    if (self%legend_==.false.) write ( file_unit, '(a)' ) 'unset key'
    if (self%legend_==.true.) write ( file_unit, '(a)' ) 'set key right top '
    
    if (self%grids_==.true.) then 
        write ( file_unit, '(a)' ) ''
        write ( file_unit, '(a)' ) '# Grid'
        write ( file_unit, '(a)' ) 'set grid'
        write ( file_unit, '(a)' ) 'set style line 12 lc rgb "#808080" lt 0 lw 1'
        write ( file_unit, '(a)' ) 'set grid back ls 12'
    end if 
    
    write ( file_unit, '(a)' ) 'set mxtics 1'
    write ( file_unit, '(a)' ) 'set mytics 1'     
            
    write ( file_unit, '(a)' ) 'set title '//'"'//trim(self%title_)//'"'
            
    write ( file_unit, '(a)' ) 'set xlabel '//'"'//trim(self%xlabel_)//'"'
    write ( file_unit, '(a)' ) 'set ylabel '//'"'//trim(self%ylabel_)//'"'
 
    if(self%xaxismode_==2) then 
            write ( file_unit, '(a)' ) ''
            write ( file_unit, '(a)' ) '# Set time formating for ox-axis'
            write ( file_unit, '(a)' ) 'set xdata time'
            write ( file_unit, '(a)' ) 'set timefmt "%s"'
            write ( file_unit, '(a)' ) 'set format x '//'"'//trim(self%datefmt_)//'"' 
    end if 
    
    if ( abs(self%uppermarginy_)>0.00001d0 .or. abs(self%lowermarginy_)>0.00001d0 ) then
            write ( file_unit, '(a)' ) ''
            write ( file_unit, '(a)' ) '# Set range of oy-axis'
            write(my_text_tmp1, '(e32.16,a1,e32.16)') miny,':',maxy
		    write ( file_unit, '(a)' ) 'set yrange ['//trim(my_text_tmp1)//']'
		    write ( file_unit, '(a)' ) 'set size square'   
    end if
 
    write ( file_unit, '(a)' ) '# User issued commands'
    
    if(self%numcommands_>0) then
            do i=1,self%numcommands_
                write ( file_unit, '(a)' ) self%commands_(i)
            end do
    end if 
    
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Labels'
    if(self%numlabels_>0) then
            do i=1,self%numlabels_
 
                write ( file_unit, '(a,i2,a, e32.16,a,e32.16,a, i8, a )' ) 'set label ', i, ' at  ', self%labels_(i)%xpos , ', ', &
                                            & self%labels_(i)%ypos , ' '//'"'//trim(self%labels_(i)%label)//'"'// &
                                                & ' center rotate by ', self%labels_(i)%rotation, ' front' // ' tc rgb ' //'"'//trim(self%labels_(i)%color)//'"'
 
            end do
    end if 
    
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Arrows'
    if(self%numarrows_>0) then
            do i=1,self%numarrows_
 
                write ( file_unit, '(a,i2,a)' ) 'set style arrow ', i, ' head filled lw 2'
                write ( file_unit, '(a,i2)' ) 'set arrow arrowstyle ', i
 
                write ( file_unit, '(a,i2,a,e32.16,a,e32.16,a,e32.16,a,e32.16,a)' ) 'set arrow ', i, ' from first  ',self%arrows_(i)%from_xpos,', ',self%arrows_(i)%from_ypos,' to ', &
                                                                              self%arrows_(i)%to_xpos, ', ',  self%arrows_(i)%to_ypos,  &
                                                                              & ' lt 2 linecolor rgb '  //'"'//trim(self%arrows_(i)%color)//'"'// ' lw 2 front size .3, 15 '
            end do
    end if 
    
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Plots'
    do j=1,ny
            if ( DataY(j)%userSuppliedStyle==.true. )  then
                
                if(ny==1) then 
                    
                        write ( file_unit, '(a,i2,a,i2,a,i2,a)' ) 'plot ' // '"'// trim (data_file_name) //'"'// &  
		                & ' using ',  1, ':', j+1, '  with ' // my_line_style(j)  //  &
                        & ' linecolor rgb "' // trim(DataY(j)%linecolor) //'"'  // ' linewidth '// trim(DataY(j)%linewidth) //' title '//'"'//trim(DataY(j)%title)//'"' 
 
                else
        
                    if(j==1) then 
                        write ( file_unit, '(a,i2,a,i2,a,i2,a)' ) 'plot ' // '"'// trim (data_file_name) // '"'// &  
		                             &  ' using ',  1, ':', j+1, '  with ' // trim(my_line_style(j))  //  &
                                         &  ' linecolor rgb "' // trim(DataY(j)%linecolor) //'"'  // ' linewidth ' // &
                                            trim(DataY(j)%linewidth) // ' title ' // '"'// trim(DataY(j)%title) // '"'// ',\' 
                    else
                        if(j<ny .and. j>1) then 
		                        write ( file_unit, '(a,i2,a,i2,a,i2,a)' ) '     ' // '"'// trim (data_file_name) // '"'// &  
		                        &   ' using ',  1, ':', j+1, '  with ' //  trim(my_line_style(j))  //  &
                                &    ' linecolor rgb "' // trim(DataY(j)%linecolor) //'"'  // ' linewidth '// &
                                    & trim(DataY(j)%linewidth) // ' title ' // '"'// trim(DataY(j)%title) // '"'// ',\' 
                        else
                                write ( file_unit, '(a,i2,a,i2,a,i2,a)' ) '     ' // '"'// trim (data_file_name) // '"'// &  
		                        &   ' using ',  1, ':', j+1, '  with ' //  trim(my_line_style(j))  //  &
                                &    ' linecolor rgb "' // trim(DataY(j)%linecolor) //'"' // ' linewidth '// &
                                     trim(DataY(j)%linewidth) // ' title ' // '"'// trim(DataY(j)%title)  // '"' 
                         end if
        
                        end if
                    end if 
                
            else
                
                if(ny==1) then 
                        write ( file_unit, '(a,i2,a,i2,a,i2,a)' ) 'plot ' // '"'// trim (data_file_name) //'"'// &  
		                & ' u ',  1, ':', j+1, '  with lines'  // ' linetype ', j, & 
                        & ' lw '// trim(DataY(j)%linewidth) //' title '//'"'//trim(DataY(j)%title)//'"' 
                else
        
                    if(j==1) then 
                        write ( file_unit, '(a,i2,a,i2,a,i2,a)' ) 'plot ' // '"'// trim (data_file_name) // '"'// &  
		                    &  ' u ',  1, ':', j+1, '  with lines ' // ' linetype ', j, & 
                            &   ' lw '// trim(DataY(j)%linewidth) // ' title ' // '"'// trim(DataY(j)%title) // '"'// ',\' 
                    else
                        if(j<ny ) then 
		                        write ( file_unit, '(a,i2,a,i2,a,i2,a)' ) '         ' // '"'// trim (data_file_name) // '"'// &  
		                        &   ' u ',  1, ':', j+1, '  with lines '// ' linetype ', j, & 
                                &   ' lw '// trim(DataY(j)%linewidth) // ' title ' // '"'// trim(DataY(j)%title) // '"'// ',\' 
                        else
                                write ( file_unit, '(a,i2,a,i2,a,i2,a)' ) '        "' // trim (data_file_name) // '"'// &  
		                        &   ' u ',  1, ':', j+1, ' with lines ' // ' linetype ', j, &  
                                &   ' lw '// trim(DataY(j)%linewidth) // ' title ' // '"'// trim(DataY(j)%title)  // '"' 
                        end if
                    end if 
                end if
            end if 
    end do 

    deallocate(xdat); deallocate(ydat) ; deallocate(arrayn); deallocate(my_line_style)  

    if( pdfcairo==.true. .and. terminalme==1 .and. rungnu==.true. ) then !compile 
            close ( unit = file_unit )
            call run_gnuplot_cairo (command_file_name, tex_file_name, data_file_name, stealth ) 
    else 
            if(rungnu==.true.) then 
                if( terminalme==0 ) then 
                    write ( file_unit, '(a)' ) '# Pause and quit gnuplot'
                    write ( file_unit, '(a)' ) 'pause -1 "Press a key..."'
                    write ( file_unit, '(a)' ) 'q'
                    close ( unit = file_unit )
                    call run_gnuplot ( 0, command_file_name, output_file_name, data_file_name, stealth )
                else
                    close ( unit = file_unit )
                    call run_gnuplot ( 1, command_file_name, output_file_name, data_file_name, stealth )
                end if 
            else
                close ( unit = file_unit )
            end if 
    end if

end subroutine gnuplot2D
 
subroutine gnusubplot2D (self, fontsize, numrows, numcols, DataX, DataY, terminal, label, stealthin ) 

	implicit none
    integer, parameter		    :: Nc=32, Nclong=128
    class(GnuFigureMaker)       :: self
    integer                     :: fontsize
    integer                     :: numrows
    integer                     :: numcols
    type(GnuDataSeries)         :: DataX(:) 
    type(GnuDataSeries)         :: DataY(:) 
    character(len=*), optional  :: terminal
    character(len=*), optional  :: label
    logical, optional           :: stealthin
    character(len=Nclong)           :: my_text_tmp1, my_text_tmp2, my_text_tmp3, my_text_tmp4
	character(len=Nclong)	    :: output_file_name, data_file_name, command_file_name, tex_file_name
 
    character(len=Nc), dimension(:), allocatable   :: my_line_style 
    character(len=Nc), dimension(:,:), allocatable :: xdat, ydat
    integer, dimension(:), allocatable :: arrayy 

    real*8 :: maxy, miny, minx, maxx
    
    integer   :: j, i, ios, file_unit, nmax, nx, ny, terminalme 
    logical   :: stealth 
  
    if( self%dimension_ .ne. 2 ) then 
        print*, 'Error in gnuplotClass: dimension specification inconsistent!' 
        return
    end if 
    
    if( present(stealthin) ) then 
        stealth=stealthin
    else 
        stealth=stealthmode
    end if 

    !check if output file present 
    if ( present(terminal) ) then 
        if( terminal == 'screen' ) then 
            terminalme=0    
        else 
            terminalme=1
            if( .not. present(label)) then 
           		    print *,'Fatal error in gnuplotter: Must specify output goes to files!'
		            return  
            end if
            
            output_file_name='gnu-fig-'//trim(label)//'.'//trim(terminal)
                        
        end if 
    else
        terminalme=0
    end if
    
 !determine presence of input identifier 
	if (present(label)) then
		data_file_name='gnu-dat-'//trim(label)//'.txt'
		command_file_name='gnu-com-'//trim(label)//'.txt'		
        tex_file_name='gnu-cairo-'//trim(label)//'.tex'
	else
		data_file_name='temp-data-'//trim(timestamp())//'.txt'
		command_file_name='temp-command-'//trim(timestamp())//'.txt'
        tex_file_name='gnu-cairo-'//trim(timestamp())//'.tex"'
        stealth=.true.
    end if
 
!open file
     
	call get_unit(file_unit)	
	if (file_unit==0) then 
		print *,'Fatal error in gnuplotter: Could not get a free FORTRAN unit!'
		return
	end if
	open (unit=file_unit, file=data_file_name, status='replace', iostat=ios)	
	if (ios/=0) then 
		print *,'Fatal error in gnuplotter: Could not open data file!'
		return
    end if

    nx=size(DataX); ny=size(DataY) ; nmax=0
            
    if(nx .ne. ny) then 
       print*, 'Error in gnuplotClass: dimensions of DataX and DataY do not match!' 
       return
    end if
        
    allocate( arrayy(1:ny), my_line_style(1:ny)  )  
    
    do j=1,ny
        arrayy(j)   = DataY(j)%size+DataY(j)%offset
    end do 
    
    do j=1,nx
        
        if(DataX(j)%size  .ne. arrayy(j))  then 
        print*, 'Error in gnuplotClass: dimensions of DataX and DataY do not match!' 
        return
        end if
        
    end do   

    nmax=maxval(arrayy(1:ny))
 
    !create char output arrays
    
    allocate(xdat(1:nmax,1:ny)) 
    allocate(ydat(1:nmax,1:ny)) 

    maxy=0.; miny=0.
 
    do j=1,ny
            
            select case( DataY(j) % dataType )
            
                case( DATA_SINGLE_PRECISION_TYPE )
                
                    do i=1,nmax
                        
                            if(i<=arrayy(j) .and. i>DataY(j)%offset ) then 
                                write (ydat(i,j),'(e32.16)') DataY(j)%singleValues(i+DataY(j)%offset) 
                                maxy=max(maxy,dble(DataY(j)%singleValues(i+DataY(j)%offset)  ))
                                miny=min(miny,dble(DataY(j)%singleValues(i+DataY(j)%offset)  ))
                            else 
                                write (ydat(i,j),'(a)') ''
                            end if
                      end do 
        
                case( DATA_DOUBLE_PRECISION_TYPE )
                    
                     do i=1,nmax
                         
                            if(i<=arrayy(j) .and. i>DataY(j)%offset) then 
                                write (ydat(i,j),'(e32.16)') DataY(j)%doubleValues(i+DataY(j)%offset) 
                                maxy=max(maxy, DataY(j)%doubleValues(i+DataY(j)%offset)  )
                                miny=min(miny, DataY(j)%doubleValues(i+DataY(j)%offset)  )
                            else 
                                write (ydat(i,j),'(a)') ''
                            end if 
                            
                      end do 
            
                case( DATA_INTEGER_TYPE )  
                        
                      do i=1,nmax
                            
                        if(i<=arrayy(j) .and. i>DataY(j)%offset) then 
                                write (ydat(i,j),'(I8)') DataY(j)%integerValues(i+DataY(j)%offset) 
                                maxy=max(maxy,dble(DataY(j)%integerValues(i+DataY(j)%offset)   ))
                                miny=min(miny,dble(DataY(j)%integerValues(i+DataY(j)%offset)   ))
                         else 
                                write (ydat(i,j),'(a)') ''
                         end if   
                        
                      end do 
 
                 case default
                            print*, 'Error in gnuplotter: Input data type illegal!'
                 end select   
    end do 
    
    maxy=maxy+self % uppermarginy_
    miny=miny-self % lowermarginy_

    maxy=0.; miny=0.
 
    do j=1,ny
            
            select case( DataX(j) % dataType )
            
                case( DATA_SINGLE_PRECISION_TYPE )
                
                    do i=1,nmax
                        
                        if(i<=arrayy(j) .and. i>DataX(j)%offset) then 
                                write (xdat(i,j),'(e32.16)') DataX(j)%singleValues(i) 
                                maxx=max(maxx,dble(DataX(j)%singleValues(i) ))
                                minx=min(minx,dble(DataX(j)%singleValues(i) ))
                         else 
                                write (xdat(i,j),'(a)') ''
                         end if  
                         
                      end do 
        
                case( DATA_DOUBLE_PRECISION_TYPE )
                    
                     do i=1,nmax
                         
                        if(i<=arrayy(j) .and. i>DataX(j)%offset) then 
                                write (xdat(i,j),'(e32.16)') DataX(j)%doubleValues(i) 
                                maxx=max(maxx,dble(DataX(j)%doubleValues(i) ))
                                minx=min(minx,dble(DataX(j)%doubleValues(i) ))
                         else 
                                write (xdat(i,j),'(a)') ''
                         end if   
                            
                      end do 
            
                case( DATA_INTEGER_TYPE )  
                        
                      do i=1,nmax
                            
                        if(i<=arrayy(j) .and. i>DataX(j)%offset) then 
                                write (xdat(i,j),'(i8)') DataX(j)%integerValues(i) 
                                maxx=max(maxx,dble(DataX(j)%integerValues(i) ))
                                minx=min(minx,dble(DataX(j)%integerValues(i) ))
                         else 
                                write (xdat(i,j),'(a)') ''
                         end if   
                        
                      end do 
                      
                  case( DATA_TIME_TYPE )  
                        
                      do i=1,nmax
                            
                        if(i<=arrayy(j) .and. i>DataX(j)%offset) then 
                                write (xdat(i,j),'(e32.16)') DataX(j)%timeValues(i) 
                                maxx=max(maxx,dble(DataX(j)%timeValues(i) ))
                                minx=min(minx,dble(DataX(j)%timeValues(i) ))
                         else 
                                write (xdat(i,j),'(a)') ''
                         end if   
                        
                      end do 
 
                 case default
                            print*, 'Error in gnuplotter: Input data type illegal!'
                 end select   
    end do 
    
    maxx=maxx+self % uppermarginx_
    minx=minx-self % lowermarginx_
 
    !write data to data_file - the gnuplot will read this data later 
 
	do i=1,nmax
        do j=1,ny-1
		    write (file_unit,'(a)',advance='no') trim(xdat(i,j))//', ' 
            write (file_unit,'(a)', advance='no') trim(ydat(i,j))//', '
        end do 
            write (file_unit,'(a)',advance='no') trim(xdat(i,j))//', ' 
            write (file_unit,'(a)',advance='yes') trim(ydat(i,ny)) 
	end do
	write (file_unit,'(a)')
 
	close (unit=file_unit) !data file has been prepared
    
    !prepare command file

 
	call get_unit(file_unit)
	if (file_unit==0) then 
		print *,'Fatal error in gnuplotter: Could not get a free FORTRAN unit!'
		return
	end if
	open (unit=file_unit, file=command_file_name, status='replace',	iostat=ios)
	if (ios/=0) then 
		print *,'Fatal error in gnuplotter: Could not open the terminal command file!'
		return
    end if	
    
    !write gnu commands to command file - gnuplot will read this later 
    
    !read optional setttings
 
    do j=1,ny
	    if ( DataY(j)%userSuppliedStyle==.true. )  then
            if ((DataY(j)%linestyle(1:1)=='.')) then
                my_line_style(j)='points pointtype ' // pointBank(min(11,j)) 
            elseif ((DataY(j)%linestyle(1:1)==':')) then    
		        my_line_style(j)='linespoints pointtype ' // pointBank(min(11,j)) 
	        else
		        my_line_style(j)='lines'
	        end if
        end if 
    end do 
 
    write ( file_unit, '(a)' ) 'reset'
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Datafile seperator'
    write ( file_unit, '(a)' ) 'set datafile separator ","'
 
    if( pdfcairo==.true. .and. terminalme==1 ) then 
            write ( file_unit, '(a)' ) ''
            write ( file_unit, '(a)' ) '# Terminal '
            write ( file_unit, '(a,i2,a)' ) 'set terminal cairolatex standalone pdf size 16cm,10.5cm font ",',fontsize,'" dashed transparent \'
            if(color==.false.) write ( file_unit, '(a)' ) ' monochrome header ''\newcommand{\hl}[1]{\setlength{\fboxsep}{0.75pt}\colorbox{white}{#1}}'' '
            if(color==.true.) write ( file_unit, '(a)' ) 'color header ''\newcommand{\hl}[1]{\setlength{\fboxsep}{0.75pt}\colorbox{white}{#1}}'''
            write ( file_unit, '(a)' ) 'set output '//'"'//trim(tex_file_name)//'"'

    elseif( pdfcairo==.false. .and. terminalme==1 ) then
        
        write ( file_unit, '(a)' ) 'set terminal pdf font "Verdana,8"'
        
    else  
            write ( file_unit, '(a)' ) 'set terminal  wxt enhanced font "Verdana,8"'
    end if
    
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Legend'
    if (self%legend_==.false.) write ( file_unit, '(a)' ) 'unset key'
    if (self%legend_==.true.) write ( file_unit, '(a)' ) 'set right left top '
    
    write ( file_unit, '(a)' ) '# Figure title'
    write ( file_unit, '(a)' ) 'set title '//'"'//trim(self%title_)//'"'
    
    if ( abs(self%uppermarginy_)>0.00001d0 .or. abs(self%lowermarginy_)>0.00001d0 ) then
            write ( file_unit, '(a)' ) '# Range for oy-axis'
            write(my_text_tmp1, '(e32.16,a1,e32.16)') miny,':',maxy
		    write ( file_unit, '(a)' ) 'set yrange ['//trim(my_text_tmp1)//']'
		    write ( file_unit, '(a)' ) 'set size square'   
    end if 
    
    if ( abs(self%uppermarginx_)>0.00001d0 .or. abs(self%lowermarginx_)>0.00001d0 ) then
            write ( file_unit, '(a)' ) '# Range for ox-axis'
            write(my_text_tmp1, '(f15.7,a1,f15.7)') minx,':',maxx
		    write ( file_unit, '(a)' ) 'set xrange ['//trim(my_text_tmp1)//']'
		    write ( file_unit, '(a)' ) 'set size square'   
    end if 
                
    write ( file_unit, '(a)' ) '# User issued commands'
     if(self%numcommands_>0) then
            do i=1,self%numcommands_
                write ( file_unit, '(a)' ) self%commands_(i)
            end do
     end if 
     
     if (self%grids_==.true.) then 
        write ( file_unit, '(a)' ) ''
        write ( file_unit, '(a)' ) '# Grid'
        write ( file_unit, '(a)' ) 'set grid'
        write ( file_unit, '(a)' ) 'set style line 12 lc rgb "#808080" lt 0 lw 1'
        write ( file_unit, '(a)' ) 'set grid back ls 12'
     end if 

    !multiplot starts here 
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Plots'
    write ( file_unit, '(a, i2, a, i2, a)' ) 'set multiplot layout ', numrows, ', ', numcols,' title  '//'"'//trim(self%title_)//'"'
 
    if (ny==1 .or. ny .ne. numcols*numrows )  then 
        print*, 'Error in gnuplot multi: too few data series provided for the requested multiplot!'
        return
    end if  
            
     do j=1,ny
         
            if ( DataY(j)%userSuppliedStyle==.true. )  then
 
                        write ( file_unit, '(a, i2, a, i2, a)' ) 'set title '//'"'//trim(DataY(j)%title)//'"'
                        write ( file_unit, '(a)' ) 'set xlabel '//'"'//trim(DataX(j)%unit)//'"'
                        write ( file_unit, '(a)' ) 'set ylabel '//'"'//trim(DataY(j)%unit)//'"'
                        write ( file_unit, '(a)' ) 'unset key' 
                        
                            if(DataX(j)%dataType==DATA_TIME_TYPE) then !set time format
                                write ( file_unit, '(a)' ) 'set xdata time'
                                write ( file_unit, '(a)' ) 'set timefmt "%s"'
                                write ( file_unit, '(a)' ) 'set format x '//'"'//trim(self%datefmt_)//'"' 
                            end if 
 
                        
                        write ( file_unit, '(a,i2,a,i2,a,i2,a)' ) 'plot ' // '"'// trim (data_file_name) // '"'// &  
		                             &  ' using ',  2*(j-1)+1, ':', 2*(j-1)+2, '  with ' // trim(my_line_style(j))  //  &
                                     &  ' linecolor rgb "' // trim(DataY(j)%linecolor) //'"'  // ' linewidth ' // &
                                            trim(DataY(j)%linewidth) // ' notitle ' !// '"'// trim(DataY(j)%title) // '"' 

            else
                
                        write ( file_unit, '(a, i2, a, i2, a)' ) 'set title '//'"'//trim(DataY(j)%title)//'"'
                        write ( file_unit, '(a)' ) 'set xlabel '//'"'//trim(DataX(j)%unit)//'"'
                        write ( file_unit, '(a)' ) 'set ylabel '//'"'//trim(DataY(j)%unit)//'"'
                        write ( file_unit, '(a)' ) 'unset key'              
                        
                            if(DataX(j)%dataType==DATA_TIME_TYPE) then !set time format
                                write ( file_unit, '(a)' ) 'set xdata time'
                                write ( file_unit, '(a)' ) 'set timefmt "%s"'
                                write ( file_unit, '(a)' ) 'set format x '//'"'//trim(self%datefmt_)//'"' 
                            end if 
                        
                        write ( file_unit, '(a,i2,a,i2,a,i2,a)' ) 'plot ' // '"'// trim (data_file_name) // '"'// &  
		                    &  ' u ',  2*(j-1)+1, ':', 2*(j-1)+2, '  w l ' // ' lt ', j, & 
                            &   ' lw '// trim(DataY(j)%linewidth) // ' notitle ' 
                        
 
            end if 
    end do 
             
    write ( file_unit, '(a)' ) 'unset multiplot '
    
    deallocate(xdat); deallocate(ydat) ; deallocate(arrayy); deallocate(my_line_style)  

    if( pdfcairo==.true. .and. terminalme==1 .and. rungnu==.true. ) then !compile 
            close ( unit = file_unit )
            call run_gnuplot_cairo (command_file_name, tex_file_name, data_file_name, stealth ) 
    else 
            if(rungnu==.true.) then 
                if( terminalme==0 ) then 
                    write ( file_unit, '(a)' ) 'pause -1 "Press a key..."'
                    write ( file_unit, '(a)' ) 'q'
                    close ( unit = file_unit )
                    call run_gnuplot ( 0, command_file_name, output_file_name, data_file_name, stealth )
                else
                    close ( unit = file_unit )
                    call run_gnuplot ( 1, command_file_name, output_file_name, data_file_name, stealth )
                end if 

            else
                close ( unit = file_unit )
            end if 
            
    end if
    
end subroutine gnusubplot2D
 
subroutine gnuplot3D (self, DataX, DataY, DataXYZ, terminal, label, stealthin ) 

	implicit none
    integer, parameter		    :: Nc=32, Nclong=128
    class(GnuFigureMaker)       :: self
 
    type(GnuDataSeries)         :: DataX 
    type(GnuDataSeries)         :: DataY 
    
    type(GnuDataSeries)         :: DataXYZ 
    
    character(len=*), optional  :: terminal
    character(len=*), optional  :: label
    logical, optional           :: stealthin

    character(len=Nclong)           :: my_text_tmp1, my_text_tmp2, my_text_tmp3, my_text_tmp4, my_text_tmp5, my_text_tmp6
	character(len=Nclong)	    :: output_file_name, data_file_name, command_file_name, tex_file_name
 
    real*8 :: maxy, minx, maxz, maxx, minz, miny
    
    integer   :: j, i,  ios, file_unit, nmax, nx, ny, terminalme 
    logical   :: stealth 
    
    character(len=Nc), dimension(:,:), allocatable :: xdat, ydat, zdat
       
    !------------------------------------------------------------------- 
    
    if( self%dimension_ .ne. 3 )  then 
        print*, 'Error in gnuplotClass 3D: dimension specification inconsistent!' 
        return
        end if 
    
    if( present(stealthin) ) then 
        stealth=stealthin
    else 
        stealth=stealthmode
    end if 

    !check if output file present 
    if ( present(terminal) ) then 
        if( terminal == 'screen' ) then 
            terminalme=0    
        else 
            terminalme=1
            if( .not. present(label)) then 
           		    print *,'Fatal error in gnuplotter 3D: Must specify output goes to files!'
		            return  
            end if
            
            output_file_name='gnu_fig_'//trim(label)//'.'//trim(terminal)
                        
        end if 
    else
        terminalme=0
    end if
    
 !determine presence of input identifier 
	if (present(label)) then
		data_file_name='gnu-dat3D-'//trim(label)//'.txt'
		command_file_name='gnu-com3D-'//trim(label)//'.txt'		
        tex_file_name='gnu-cairo3D-'//trim(label)//'.tex'
	else
		data_file_name='temp-dat3D-'//trim(timestamp())//'.txt'
		command_file_name='temp-com3D-'//trim(timestamp())//'.txt'
        tex_file_name='gnu-cairo3D-'//trim(timestamp())//'.tex"'
        stealth=.true.
    end if
 
!open file
    
	call get_unit(file_unit)	
	if (file_unit==0) then
		print *,'Fatal error in gnuplotter 3D: Could not get a free FORTRAN unit!'
		return
	end if
	open (unit=file_unit, file=data_file_name, status='replace', iostat=ios)	
	if (ios/=0) then
		print *,'Fatal error in gnuplotter 3D: Could not open data file!'
		return
    end if

    nx=DataXYZ%size
    ny=DataXYZ%size2
 
    if( DataX%size .NE. nx) then 
        print*, 'Fatal error in gnuplotter 3D: Array size mismatch!'
        return
        end if 
    if( DataY%size .NE. ny)  then 
        print*, 'Fatal error in gnuplotter 3D: Array size mismatch!'
        return
        end if 

    allocate (xdat(1:nx,1:ny))
    allocate (ydat(1:nx,1:ny))
    allocate (zdat(1:nx,1:ny))
    
    maxy=0.; maxz=0.; maxx=0.
    miny=0.; minz=0.; miny=0.
 
    select case( DataX % dataType )
            
                case( DATA_SINGLE_PRECISION_TYPE )

                     do i=1,nx
                            do j=1,ny
                                write ( xdat(i,j),'(e32.16)') DataX%singleValues(i) 
                            end do 
                     end do 
        
                case( DATA_DOUBLE_PRECISION_TYPE )
                    
                     do i=1,nx
                            do j=1,ny
                                write ( xdat(i,j),'(e32.16)') DataX%doubleValues(i) 
                            end do 
                        end do  
            
                case( DATA_INTEGER_TYPE )  
                        
                        do i=1,nx
                            do j=1,ny
                                    write ( xdat(i,j),'(i8)') DataX%integerValues(i ) 
                            end do 
                        end do 
 
                 case default
                 print*, 'Error in gnuplotter: Illegal specification of input data type or data entry is empty!'
   
    end select   
 
    select case( DataY % dataType )
            
                case( DATA_SINGLE_PRECISION_TYPE )

                     do i=1,nx
                            do j=1,ny
                                write ( ydat(i,j),'(f18.6)') DataY%singleValues(j) 
                            end do 
                     end do 
        
                case( DATA_DOUBLE_PRECISION_TYPE )
                    
                     do i=1,nx
                            do j=1,ny
                                write ( ydat(i,j),'(e32.16)') DataY%doubleValues(j) 
                            end do 
                        end do  
            
                case( DATA_INTEGER_TYPE )  
                        
                        do i=1,nx
                            do j=1,ny
                                    write ( ydat(i,j),'(i8)') DataY%integerValues(j) 
                            end do 
                        end do 
 
                 case default
                 print*, 'Error in gnuplotter: Illegal specification of input data type or data entry is empty!'
   
    end select  
    
    select case( DataXYZ % dataType )
            
                case( DATA_SINGLE_PRECISION_TYPE )

                     do i=1,nx
                            do j=1,ny
                                write (zdat(i,j),'(e32.16)') DataXYZ%singleValues2(i,j)
                                maxx=max(maxx,dble(DataX%singleValues(i )  ))
                                minx=min(minx,dble(DataX%singleValues(i )  ))
                                maxy=max(maxy,dble(DataY%singleValues(j )  ))
                                miny=min(miny,dble(DataY%singleValues(j )  ))
                                maxz=max(maxz,dble(DataXYZ%singleValues2(i,j)  ))
                                minz=min(minz,dble(DataXYZ%singleValues2(i,j)  ))
                            end do 
                            write (file_unit,'(a)')
                     end do 
        
                case( DATA_DOUBLE_PRECISION_TYPE )
                    
                     do i=1,nx
                            do j=1,ny
                                write ( zdat(i,j),'(e32.16)') DataXYZ%doubleValues2(i,j)
                                maxx=max(maxx,dble(DataX%doubleValues(i )))
                                minx=min(minx,dble(DataX%doubleValues(i )))
                                maxy=max(maxy,dble(DataY%doubleValues(j )))
                                miny=min(miny,dble(DataY%doubleValues(j)))
                                maxz=max(maxz,dble(DataXYZ%doubleValues2(i,j)))
                                minz=min(minz,dble(DataXYZ%doubleValues2(i,j)))
                            end do 
                            write (file_unit,'(a)')
                        end do  
            
                case( DATA_INTEGER_TYPE )  
                        
                        do i=1,nx
                            do j=1,ny
                                    write ( zdat(i,j),'(i8)') DataXYZ%integerValues2(i,j)
                                    maxx=max(maxx,dble(DataX%integerValues(i)))
                                    minx=min(minx,dble(DataX%integerValues(i)))
                                    maxy=max(maxy,dble(DataY%integerValues(j)))
                                    miny=min(miny,dble(DataY%integerValues(j)))
                                    maxz=max(maxz,dble(DataXYZ%integerValues2(i,j)))
                                    minz=min(minz,dble(DataXYZ%integerValues2(i,j)))
                            end do 
                            write (file_unit,'(a)')
                        end do 
 
                 case default
                 print*, 'Error in gnuplotter: Illegal specification of input data type or data entry is empty!'
   
    end select   

    !write (file_unit,'(a)')  '# '//trim(DataX%title)//'   '//trim(DataY%title)//'   '//trim(DataXYZ%title)  
    do i=1,nx
        do j=1,ny                              
        write (file_unit,'(a)') xdat(i,j)//', '//ydat(i,j)//', '//zdat(i,j)                         
        end do 
        write (file_unit,'(a)')
    end do          
         
	close (unit=file_unit) !data file has been prepared
    
    maxy=maxy+self % uppermarginy_ !take max min for range bounds
    miny=miny-self % lowermarginy_
    maxz=maxz+self % uppermarginz_
    minz=minz-self % lowermarginz_
    maxx=maxx+self % uppermarginx_
    minx=minx-self % lowermarginx_     

    !prepare command file
	call get_unit(file_unit)
	if (file_unit==0) then
		print *,'Fatal error in gnuplotter: Could not get a free FORTRAN unit!'
		return
	end if
	open (unit=file_unit, file=command_file_name, status='replace',	iostat=ios)
	if (ios/=0) then
		print *,'Fatal error in gnuplotter: Could not open the terminal command file!'
		return
    end if	
 
    !read optional setttings
    
    write ( file_unit, '(a)' ) 'reset'
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Datafile seperator'
    write ( file_unit, '(a)' ) 'set datafile separator ","'
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Terminal and output files'
    if( pdfcairo==.true. .and. terminalme==1 ) then 
            write ( file_unit, '(a)' ) 'set terminal cairolatex standalone pdf size 16cm,10.5cm dashed transparent \'
            if(color==.false.) write ( file_unit, '(a)' ) ' monochrome header ''\newcommand{\hl}[1]{\setlength{\fboxsep}{0.75pt}\colorbox{white}{#1}}'' '
            if(color==.true.) write ( file_unit, '(a)' ) 'color header ''\newcommand{\hl}[1]{\setlength{\fboxsep}{0.75pt}\colorbox{white}{#1}}'''
            write ( file_unit, '(a)' ) 'set output '//'"'//trim(tex_file_name)//'"'

    elseif( pdfcairo==.false. .and. terminalme==1 ) then
        
        write ( file_unit, '(a)' ) 'set terminal pdf '
        
    else  
            write ( file_unit, '(a)' ) 'set terminal  wxt enhanced '
    end if
    
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Grids'
    if (self%grids_==.true.) write ( file_unit, '(a)' ) 'set grid'
    
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Figure title'
    write ( file_unit, '(a)' ) 'set title '//'"'//trim(self%title_)//'"'

    
    if ( abs(self%uppermarginz_)>0.00001d0 .or. abs(self%lowermarginz_)>0.00001d0 ) then
        write ( file_unit, '(a)' ) ''
        write ( file_unit, '(a)' ) '# Range for oz axis'
            write(my_text_tmp1, '(e32.16,a1,e32.16)') minz,':',maxz
		    write ( file_unit, '(a)' ) 'set zrange ['//trim(my_text_tmp1)//']'
    end if 
    
    if ( abs(self%uppermarginx_)>0.00001d0 .or. abs(self%lowermarginx_)>0.00001d0 ) then
        write ( file_unit, '(a)' ) ''
        write ( file_unit, '(a)' ) '# Range for ox axis'
            write(my_text_tmp1, '(e32.16,a1,e32.16)') minx,':',maxx
		    write ( file_unit, '(a)' ) 'set xrange ['//trim(my_text_tmp1)//']'
    end if 
    
    if ( abs(self%uppermarginy_)>0.00001d0 .or. abs(self%lowermarginy_)>0.00001d0 ) then
        write ( file_unit, '(a)' ) ''
        write ( file_unit, '(a)' ) '# Range for oy axis'
            write(my_text_tmp1, '(e32.16,a1,e32.16)') miny,':',maxy
		    write ( file_unit, '(a)' ) 'set yrange ['//trim(my_text_tmp1)//']'
    end if     
    
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Asis labels'
    write ( file_unit, '(a)' ) 'set xlabel '//'"'//trim(self%xlabel_)//'"'
    write ( file_unit, '(a)' ) 'set ylabel '//'"'//trim(self%ylabel_)//'"'
    write ( file_unit, '(a)' ) 'set zlabel '//'"'//trim(self%zlabel_)//'"'
    
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# No legend'
    write ( file_unit, '(a)' ) 'set nokey'
    
    
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Plot mode'
	write ( file_unit, '(a)' ) 'set palette model '// trim(default_palette)

    write ( file_unit, '(a)' ) 'set contour base'
    
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# User issued commands'
    if(self%numcommands_>0) then
            do i=1,self%numcommands_
                write ( file_unit, '(a)' ) self%commands_(i)
            end do
    end if 
    
    write ( file_unit, '(a)' ) ''  
    write ( file_unit, '(a)' ) '# Labels'
    if(self%numlabels_>0) then
            do i=1,self%numlabels_
                write(my_text_tmp1, '(e32.16)')  self%labels_(i)%xpos 
                write(my_text_tmp2, '(e32.16)')  self%labels_(i)%ypos 
                write(my_text_tmp3, '(e32.16)')  self%labels_(i)%zpos 
                write(my_text_tmp4, '(e32.16)')  self%labels_(i)%rotation
                write ( file_unit, '(a,i2,a)' ) 'set label ', i, ' at  '//trim(my_text_tmp1)//', '//trim(my_text_tmp2)//', '//trim(my_text_tmp3)//' '//'"'//trim(self%labels_(i)%label)//'"'// &
                                                & ' center rotate by '//trim(my_text_tmp3)// ' front' // ' tc rgb ' //'"'// trim(self%labels_(i)%color)//'"'
            end do
    end if 
    
    write ( file_unit, '(a)' ) ''        
    write ( file_unit, '(a)' ) '# Arrows'
    if(self%numarrows_>0) then
            do i=1,self%numarrows_
                write(my_text_tmp1, '(e32.16)')  self%arrows_(i)%from_xpos
                write(my_text_tmp2, '(e32.16)')  self%arrows_(i)%from_ypos
                write(my_text_tmp3, '(e32.16)')  self%arrows_(i)%from_zpos
                write(my_text_tmp4, '(e32.16)')  self%arrows_(i)%to_xpos
                write(my_text_tmp5, '(e32.16)')  self%arrows_(i)%to_ypos
                write(my_text_tmp5, '(e32.16)')  self%arrows_(i)%to_zpos
               
                write ( file_unit, '(a,i2,a)' ) 'set style arrow ', i, ' head filled lw 2'
                write ( file_unit, '(a,i2)' ) 'set arrow arrowstyle ', i
 
                write ( file_unit, '(a,i2,a)' ) 'set arrow ', i, ' from first  '//trim(my_text_tmp1)//', '//trim(my_text_tmp2)//', '//trim(my_text_tmp3) // &
                                                                          ' to '//trim(my_text_tmp4)//', '//trim(my_text_tmp5)//', '//trim(my_text_tmp6) // &
                                                & ' lt 2 linecolor rgb '  //'"'// self%arrows_(i)%color  //'"'// ' lw 2 front size .3, 15 '
            end do
    end if 
 
	!write ( file_unit, '(a)' ) 'set hidden3d'
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Plots'   
    
	if( pm3d==.false.) write ( file_unit, '(a)' ) 'splot "' // trim ( data_file_name ) // '" using 1:2:3 with lines palette title '//'"'//trim(DataXYZ % title)//'"' 
    
    if( pm3d==.true.) write ( file_unit, '(a)' ) 'splot "' // trim ( data_file_name ) // '" w pm3d title '//'"'//trim(DataXYZ % title)//'"' 
 
    deallocate (xdat, ydat, zdat)
        
    if( pdfcairo==.true. .and. terminalme==1 .and. rungnu==.true. ) then !compile 
            close ( unit = file_unit )
            call run_gnuplot_cairo (command_file_name, tex_file_name, data_file_name, stealth ) 
    else 
            if(rungnu==.true.) then 
                if( terminalme==0 ) then 
                    write ( file_unit, '(a)' ) 'pause -1 "Press a key..."'
                    write ( file_unit, '(a)' ) 'q'
                    close ( unit = file_unit )
                    call run_gnuplot ( 0, command_file_name, output_file_name, data_file_name, stealth )
                else
                    close ( unit = file_unit )
                    call run_gnuplot ( 1, command_file_name, output_file_name, data_file_name, stealth )
                end if 

            else
                close ( unit = file_unit )
            end if 
            
    end if
    
end subroutine gnuplot3D  

subroutine gnuplot3D_2(self, DataX, DataY, DataXYZ, andDataXYZ, terminal, label, stealthin ) 

	implicit none
    integer, parameter		    :: Nc=32, Nclong=128
    class(GnuFigureMaker)       :: self
 
    type(GnuDataSeries)         :: DataX 
    type(GnuDataSeries)         :: DataY 
    
    type(GnuDataSeries)         :: DataXYZ, andDataXYZ 
    
    character(len=*), optional  :: terminal
    character(len=*), optional  :: label
    logical, optional           :: stealthin
    character(len=Nclong)           :: my_text_tmp1, my_text_tmp2, my_text_tmp3, my_text_tmp4, my_text_tmp5, my_text_tmp6
	character(len=Nclong)	    :: output_file_name, data_file_name, data_file_name1, command_file_name, tex_file_name
 
    real*8 :: maxy, minx, maxz, maxx, minz, miny
    
    integer   :: j, i, ios, file_unit, nmax, nx, ny, terminalme 
    logical   :: stealth 

    character(len=Nc), dimension(:,:), allocatable :: xdat, ydat, zdat
    
    !------------------------------------------------------------------- 
    
    if( self%dimension_ .ne. 3 )  then 
        print*, 'Error in gnuplotClass 3D: dimension specification inconsistent!' 
        return
        end if
    
    if( present(stealthin) ) then 
        stealth=stealthin
    else 
        stealth=stealthmode
    end if 

    !check if output file present 
    if ( present(terminal) ) then 
        if( terminal == 'screen' ) then 
            terminalme=0    
        else 
            terminalme=1
            if( .not. present(label)) then 
           		    print *,'Fatal error in gnuplotter 3D: Must specify output goes to files!'
		            return  
            end if
            
            output_file_name='gnu_fig_'//trim(label)//'.'//trim(terminal)
                        
        end if 
    else
        terminalme=0
    end if
    
 !determine presence of input identifier 
	if (present(label)) then
		data_file_name='gnu-dat3D-'//trim(label)//'-1.txt'
        data_file_name1='gnu-dat3D-'//trim(label)//'-2.txt'
		command_file_name='gnu-com3D-'//trim(label)//'.txt'		
        tex_file_name='gnu-cairo3D-'//trim(label)//'.tex'
	else
		data_file_name='gnu-dat3D-'//trim(timestamp())//'-1.txt'
        data_file_name1='gnu-dat3D-'//trim(label)//'-2.txt'
		command_file_name='gnu-com3D-'//trim(timestamp())//'.txt'
        tex_file_name='gnu-cairo3D-'//trim(timestamp())//'.tex"'
        stealth=.true.
    end if
 
!open file
    
    nx=DataXYZ%size; ny=DataXYZ%size2
 
    if( DataX%size .NE. nx)  then 
        print*, 'Fatal error in gnuplotter 3D 2: Array size mismatch!'
        return
    end if 
    
    if( DataY%size .NE. ny)  then 
        print*, 'Fatal error in gnuplotter 3D 2: Array size mismatch!'
        return
    end if
    
    allocate (xdat(1:nx,1:ny)) !create database text files
    allocate (ydat(1:nx,1:ny))
    allocate (zdat(1:nx,1:ny))
    
    maxy=0.; maxz=0.; maxx=0. !reset ranges
    miny=0.; minz=0.; miny=0.
    
    !open file data_file_name1
	call get_unit(file_unit)	
	if (file_unit==0) then
		print *,'Fatal error in gnuplotClass: Could not get a free FORTRAN unit!'
		return
	end if
	open (unit=file_unit, file=data_file_name, status='replace', iostat=ios)	
	if (ios/=0) then
		print *,'Fatal error in gnuplotClass: Could not open data file 2!'
		return
    end if 
 
    select case( DataX % dataType )
            
                case( DATA_SINGLE_PRECISION_TYPE )

                     do i=1,nx
                            do j=1,ny
                                write ( xdat(i,j),'(e32.16)') DataX%singleValues(i) 
                            end do 

                     end do 
        
                case( DATA_DOUBLE_PRECISION_TYPE )
                    
                     do i=1,nx
                            do j=1,ny
                                write ( xdat(i,j),'(e32.16)') DataX%doubleValues(i) 
                            end do 

                        end do  
            
                case( DATA_INTEGER_TYPE )  
                        
                        do i=1,nx
                            do j=1,ny
                                    write ( xdat(i,j),'(i8)') DataX%integerValues(i ) 
                            end do 

                        end do 
 
                 case default
                 print*, 'Error in gnuplotter: Illegal specification of input data type or data entry is empty!'
   
    end select   
 
    select case( DataY % dataType )
            
                case( DATA_SINGLE_PRECISION_TYPE )

                     do i=1,nx
                            do j=1,ny
                                write ( ydat(i,j),'(e32.16)') DataY%singleValues(j) 
                            end do 

                     end do 
        
                case( DATA_DOUBLE_PRECISION_TYPE )
                    
                     do i=1,nx
                            do j=1,ny
                                write ( ydat(i,j),'(e32.16)') DataY%doubleValues(j) 
                            end do 

                        end do  
            
                case( DATA_INTEGER_TYPE )  
                        
                        do i=1,nx
                            do j=1,ny
                                    write ( ydat(i,j),'(i8)') DataY%integerValues(j) 
                            end do 

                        end do 
 
                 case default
                 print*, 'Error in gnuplotter: Illegal specification of input data type or data entry is empty!'
   
    end select  
    
    select case( DataXYZ % dataType )
            
                case( DATA_SINGLE_PRECISION_TYPE )

                     do i=1,nx
                            do j=1,ny
                                write (zdat(i,j),'(e32.16)') DataXYZ%singleValues2(i,j)
                                maxx=max(maxx,dble(DataX%singleValues(i )  ))
                                minx=min(minx,dble(DataX%singleValues(i )  ))
                                maxy=max(maxy,dble(DataY%singleValues(j )  ))
                                miny=min(miny,dble(DataY%singleValues(j )  ))
                                maxz=max(maxz,dble(DataXYZ%singleValues2(i,j)  ))
                                minz=min(minz,dble(DataXYZ%singleValues2(i,j)  ))
                            end do 

                     end do 
        
                case( DATA_DOUBLE_PRECISION_TYPE )
                    
                     do i=1,nx
                            do j=1,ny
                                write ( zdat(i,j),'(e32.16)') DataXYZ%doubleValues2(i,j)
                                maxx=max(maxx,dble(DataX%doubleValues(i )))
                                minx=min(minx,dble(DataX%doubleValues(i )))
                                maxy=max(maxy,dble(DataY%doubleValues(j )))
                                miny=min(miny,dble(DataY%doubleValues(j)))
                                maxz=max(maxz,dble(DataXYZ%doubleValues2(i,j)))
                                minz=min(minz,dble(DataXYZ%doubleValues2(i,j)))
                            end do 

                        end do  
            
                case( DATA_INTEGER_TYPE )  
                        
                        do i=1,nx
                            do j=1,ny
                                    write ( zdat(i,j),'(i8)') DataXYZ%integerValues2(i,j)
                                    maxx=max(maxx,dble(DataX%integerValues(i)))
                                    minx=min(minx,dble(DataX%integerValues(i)))
                                    maxy=max(maxy,dble(DataY%integerValues(j)))
                                    miny=min(miny,dble(DataY%integerValues(j)))
                                    maxz=max(maxz,dble(DataXYZ%integerValues2(i,j)))
                                    minz=min(minz,dble(DataXYZ%integerValues2(i,j)))
                            end do 

                        end do 
 
                 case default
                 print*, 'Error in gnuplotter: Illegal specification of input data type or data entry is empty!'
   
    end select   

    write (file_unit,'(a)')  '# '//trim(DataX%title)//'   '//trim(DataY%title)//'   '//trim(DataXYZ%title)
    do i=1,nx
        do j=1,ny                              
        write (file_unit,'(a)') xdat(i,j)//', '//ydat(i,j)//', '//zdat(i,j)                         
        end do 
        write (file_unit,'(a)')
    end do          
         
	close (unit=file_unit) !data file has been prepared

    if( DataXYZ%size .NE. DataX%size ) then 
        print*, 'Fatal error in gnuplotter 3D: Datatype   mismatch!'
        return
    end if
    
    if( DataXYZ%size2 .NE. DataY%size) then 
        print*,  'Fatal error in gnuplotter 3D: Datatype   mismatch!'
        return
    end if
    
    if( andDataXYZ%size .NE. DataXYZ%size) then 
        print*,  'Fatal error in gnuplotter 3D: Datatype   mismatch!'
        return
    end if
    
    if( andDataXYZ%size2 .NE. DataXYZ%size2) then 
        print*,  'Fatal error in gnuplotter 3D: Datatype   mismatch!'
        return
    end if
    
    
    !open file data_file_name1	
	call get_unit(file_unit)	
	if (file_unit==0) then
		print *,'Fatal error in gnuplotClass: Could not get a free FORTRAN unit!'
		return
	end if
	open (unit=file_unit, file=data_file_name1, status='replace', iostat=ios)	
	if (ios/=0) then
		print *,'Fatal error in gnuplotClass: Could not open data file 2!'
		return
    end if 
    
    select case( andDataXYZ % dataType )
            
                case( DATA_SINGLE_PRECISION_TYPE )

                     do i=1,nx
                            do j=1,ny
                                write (zdat(i,j),'(e32.16)') andDataXYZ%singleValues2(i,j)
                                maxx=max(maxx,dble(DataX%singleValues(i )  ))
                                minx=min(minx,dble(DataX%singleValues(i )  ))
                                maxy=max(maxy,dble(DataY%singleValues(j )  ))
                                miny=min(miny,dble(DataY%singleValues(j )  ))
                                maxz=max(maxz,dble(DataXYZ%singleValues2(i,j)  ))
                                minz=min(minz,dble(DataXYZ%singleValues2(i,j)  ))
                            end do  
                     end do 
        
                case( DATA_DOUBLE_PRECISION_TYPE )
                    
                     do i=1,nx
                            do j=1,ny
                                write ( zdat(i,j),'(e32.16)') andDataXYZ%doubleValues2(i,j)
                                maxx=max(maxx,dble(DataX%doubleValues(i )))
                                minx=min(minx,dble(DataX%doubleValues(i )))
                                maxy=max(maxy,dble(DataY%doubleValues(j )))
                                miny=min(miny,dble(DataY%doubleValues(j)))
                                maxz=max(maxz,dble(DataXYZ%doubleValues2(i,j)))
                                minz=min(minz,dble(DataXYZ%doubleValues2(i,j)))
                            end do  
                        end do  
            
                case( DATA_INTEGER_TYPE )  
                        
                        do i=1,nx
                            do j=1,ny
                                    write ( zdat(i,j),'(i8)') andDataXYZ%integerValues2(i,j)
                                    maxx=max(maxx,dble(DataX%integerValues(i)))
                                    minx=min(minx,dble(DataX%integerValues(i)))
                                    maxy=max(maxy,dble(DataY%integerValues(j)))
                                    miny=min(miny,dble(DataY%integerValues(j)))
                                    maxz=max(maxz,dble(DataXYZ%integerValues2(i,j)))
                                    minz=min(minz,dble(DataXYZ%integerValues2(i,j)))
                            end do  
                        end do 
 
                 case default
                 print*, 'Error in gnuplotter: Illegal specification of input data type or data entry is empty!'
   
    end select   

    write (file_unit,'(a)')  '# '//trim(DataX%title)//'   '//trim(DataY%title)//'   '//trim(DataXYZ%title) 
    do i=1,nx
        do j=1,ny                              
        write (file_unit,'(a)') xdat(i,j)//', '//ydat(i,j)//', '//zdat(i,j)                         
        end do 
        write (file_unit,'(a)')
    end do          
         
	close (unit=file_unit) !data file has been prepared
 
    maxy=maxy+self % uppermarginy_ !take max min for range bounds
    miny=miny-self % lowermarginy_
    maxz=maxz+self % uppermarginz_
    minz=minz-self % lowermarginz_
    maxx=maxx+self % uppermarginx_
    minx=minx-self % lowermarginx_     

    !prepare command file
	call get_unit(file_unit)
	if (file_unit==0) then
		print *,'Fatal error in gnuplotClass: Could not get a free FORTRAN unit!'
		return
	end if
	open (unit=file_unit, file=command_file_name, status='replace',	iostat=ios)
	if (ios/=0) then
		print *,'Fatal error in gnuplotClass: Could not open the terminal command file!'
		return
    end if	
 
    !read optional setttings
    
    write ( file_unit, '(a)' ) 'reset'
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# File seperator'  
    write ( file_unit, '(a)' ) 'set datafile separator ","'
    
    if( pdfcairo==.true. .and. terminalme==1 ) then 
            write ( file_unit, '(a)' ) ''
            write ( file_unit, '(a)' ) '# Terminal and output files'  
            write ( file_unit, '(a)' ) 'set terminal cairolatex standalone pdf size 16cm,10.5cm dashed transparent \'
            if(color==.false.) write ( file_unit, '(a)' ) ' monochrome header ''\newcommand{\hl}[1]{\setlength{\fboxsep}{0.75pt}\colorbox{white}{#1}}'' '
            if(color==.true.) write ( file_unit, '(a)' ) 'color header ''\newcommand{\hl}[1]{\setlength{\fboxsep}{0.75pt}\colorbox{white}{#1}}'''
            write ( file_unit, '(a)' ) 'set output '//'"'//trim(tex_file_name)//'"'

    elseif( pdfcairo==.false. .and. terminalme==1 ) then
        
        write ( file_unit, '(a)' ) 'set terminal pdf '
        
    else  
            write ( file_unit, '(a)' ) 'set terminal wxt enhanced '
    end if
    
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Legend and grids'  
    if (self%legend_==.false.) write ( file_unit, '(a)' ) 'set nokey'
    if (self%legend_==.true.) write ( file_unit, '(a)' ) 'set key'
    if (self%grids_==.true.) write ( file_unit, '(a)' ) 'set grid'
    
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Figure title'  
    write ( file_unit, '(a)' ) 'set title '//'"'//trim(self%title_)//'"'

    if ( abs(self%uppermarginz_)>0.00001d0 .or. abs(self%lowermarginz_)>0.00001d0 ) then
            write ( file_unit, '(a)' ) ''
            write ( file_unit, '(a)' ) '# Range for oz-axis'  
            write(my_text_tmp1, '(e32.16,a1,e32.16)') minz,':',maxz
		    write ( file_unit, '(a)' ) 'set zrange ['//trim(my_text_tmp1)//']'
    end if 
    
    if ( abs(self%uppermarginx_)>0.00001d0 .or. abs(self%lowermarginx_)>0.00001d0 ) then
            write ( file_unit, '(a)' ) ''
            write ( file_unit, '(a)' ) '# Range for ox-axis'  
            write(my_text_tmp1, '(e32.16,a1,e32.16)') minx,':',maxx
		    write ( file_unit, '(a)' ) 'set xrange ['//trim(my_text_tmp1)//']'
    end if 
    
    if ( abs(self%uppermarginy_)>0.00001d0 .or. abs(self%lowermarginy_)>0.00001d0 ) then
            write ( file_unit, '(a)' ) ''
            write ( file_unit, '(a)' ) '# Range for oy-axis'  
            write(my_text_tmp1, '(e32.16,a1,e32.16)') miny,':',maxy
		    write ( file_unit, '(a)' ) 'set yrange ['//trim(my_text_tmp1)//']'
    end if     
    
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Axis labels'  
    write ( file_unit, '(a)' ) 'set xlabel '//'"'//trim(self%xlabel_)//'"'
    write ( file_unit, '(a)' ) 'set ylabel '//'"'//trim(self%ylabel_)//'"'
    write ( file_unit, '(a)' ) 'set zlabel '//'"'//trim(self%zlabel_)//'"'
 
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Plot mode'  
    
	if (self%legend_==.false.) write ( file_unit, '(a)' ) 'set palette model '// trim(default_palette)

    if (self%legend_==.false.) write ( file_unit, '(a)' ) 'set contour base'
    
    write ( file_unit, '(a)' ) 'unset colorbox'
    
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# User issued commands'  
    if(self%numcommands_>0) then
            do i=1,self%numcommands_
                write ( file_unit, '(a)' ) self%commands_(i)
            end do
    end if 
    
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Labels'  
    if(self%numlabels_>0) then
            do i=1,self%numlabels_
                write(my_text_tmp1, '(e32.16)')  self%labels_(i)%xpos 
                write(my_text_tmp2, '(e32.16)')  self%labels_(i)%ypos 
                write(my_text_tmp3, '(e32.16)')  self%labels_(i)%zpos 
                write(my_text_tmp4, '(e32.16)')  self%labels_(i)%rotation
                write ( file_unit, '(a,i2,a)' ) 'set label ', i, ' at  '//trim(my_text_tmp1)//', '//trim(my_text_tmp2)//', '//trim(my_text_tmp3)//' '//'"'//trim(self%labels_(i)%label)//'"'// &
                                             & ' center rotate by '//trim(my_text_tmp3)// ' front' // ' tc rgb ' //'"'// trim(self%labels_(i)%color)//'"'
            end do
    end if 
    
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Arrows'  
    if(self%numarrows_>0) then
            do i=1,self%numarrows_
                write(my_text_tmp1, '(e32.16)')  self%arrows_(i)%from_xpos
                write(my_text_tmp2, '(e32.16)')  self%arrows_(i)%from_ypos
                write(my_text_tmp3, '(e32.16)')  self%arrows_(i)%from_zpos
                write(my_text_tmp4, '(e32.16)')  self%arrows_(i)%to_xpos
                write(my_text_tmp5, '(e32.16)')  self%arrows_(i)%to_ypos
                write(my_text_tmp5, '(e32.16)')  self%arrows_(i)%to_zpos
               
                write ( file_unit, '(a,i2,a)' ) 'set style arrow ', i, ' head filled lw 2'
                write ( file_unit, '(a,i2)' ) 'set arrow arrowstyle ', i
 
                write ( file_unit, '(a,i2,a)' ) 'set arrow ', i, ' from first  '//trim(my_text_tmp1)//', '//trim(my_text_tmp2)//', '//trim(my_text_tmp3) // &
                                                                          ' to '//trim(my_text_tmp4)//', '//trim(my_text_tmp5)//', '//trim(my_text_tmp6) // &
                                                & ' lt 2 linecolor rgb '  //'"'// self%arrows_(i)%color  //'"'// ' lw 2 front size .3, 15 '
            end do
    end if 

	!write ( file_unit, '(a)' ) 'set hidden3d'
    
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Plots'  
    if (self%legend_==.false.) then 
	    if( pm3d==.false.)  write ( file_unit, '(a)' ) 'splot "' // trim ( data_file_name ) // '" using 1:2:3 with lines palette title '//'"'//trim(DataXYZ % title )//'", \'
        if( pm3d==.true.)  write ( file_unit, '(a)' ) 'splot "' // trim ( data_file_name ) // '" using 1:2:3 w pm3d title '//'"'//trim(DataXYZ % title )//'", \'
        write ( file_unit, '(a)' ) '      "' // trim ( data_file_name1 ) // '" using 1:2:3 with lines palette '//'"'//trim(andDataXYZ % title )//'"'
    else 
        if( pm3d==.false.)  write ( file_unit, '(a)' ) 'splot "' // trim ( data_file_name ) // '" using 1:2:3 with lines title '//'"'//trim(DataXYZ % title )//'", \'
        if( pm3d==.true.)  write ( file_unit, '(a)' ) 'splot "' // trim ( data_file_name ) // '" using 1:2:3 w pm3d title '//'"'//trim(DataXYZ % title )//'", \'
        write ( file_unit, '(a)' ) '      "' // trim ( data_file_name1 ) // '" using 1:2:3 with lines title '//'"'//trim(andDataXYZ % title )//'"'
    end if 
 
    deallocate(xdat); deallocate(ydat); deallocate(zdat)
    
    if( pdfcairo==.true. .and. terminalme==1 .and. rungnu==.true. ) then !compile 
            close ( unit = file_unit )
            call run_gnuplot_cairo (command_file_name, tex_file_name, data_file_name, stealth, data_file_name1 ) 
    else 
            if(rungnu==.true.) then 
                if( terminalme==0 ) then 
                    write ( file_unit, '(a)' ) 'pause -1 "Press a key..."'
                    write ( file_unit, '(a)' ) 'q'
                    close ( unit = file_unit )
                    call run_gnuplot ( 0, command_file_name, output_file_name, data_file_name, stealth, data_file_name1 )
                else
                    close ( unit = file_unit )
                    call run_gnuplot ( 1, command_file_name, output_file_name, data_file_name, stealth, data_file_name1 )
                end if 

            else
                close ( unit = file_unit )
            end if 
            
    end if
    
end subroutine gnuplot3D_2   
 
subroutine gnuplot3D_auto (self, DataXYZ, terminal, label, stealthin ) 

	implicit none
    integer, parameter		    :: Nc=32, Nclong=128
    class(GnuFigureMaker)       :: self
 
    type(GnuDataSeries)         :: DataXYZ 
    
    character(len=*), optional  :: terminal
    character(len=*), optional  :: label
    logical, optional           :: stealthin
    character(len=Nclong)           :: my_text_tmp1, my_text_tmp2, my_text_tmp3, my_text_tmp4, my_text_tmp5, my_text_tmp6
	character(len=Nclong)	    :: output_file_name, data_file_name, command_file_name, tex_file_name

    character(len=Nc), dimension(:,:), allocatable   :: xyzdat  
    
    character(len=Nc) :: my_line_style 
    
    integer :: arrayn

    real*8 ::   maxz,  minz
    
    integer   :: j, i, ios, file_unit, nmax, nx, ny, terminalme 
    logical   :: stealth 

    if( self%dimension_ .ne. 3 ) then 
        print*,  'Error in gnuplotClass 3D: dimension specification inconsistent!' 
        return
        end if
    
    if( present(stealthin) ) then 
        stealth=stealthin
    else 
        stealth=stealthmode
    end if 

    !check if output file present 
    if ( present(terminal) ) then 
        if( terminal == 'screen' ) then 
            terminalme=0    
        else 
            terminalme=1
            if( .not. present(label)) then 
           		    print *,'Fatal error in gnuplotter 3D: Must specify output goes to files!'
		            return  
            end if
            
            output_file_name='gnu-fig-'//trim(label)//'.'//trim(terminal)
                        
        end if 
    else
        terminalme=0
    end if
    
 !determine presence of input identifier 
	if (present(label)) then
		data_file_name='gnu-dat3D-'//trim(label)//'.txt'
		command_file_name='gnu-com3D-'//trim(label)//'.txt'		
        tex_file_name='gnu-cairo3D-'//trim(label)//'.tex'
	else
		data_file_name='temp-dat3D-'//trim(timestamp())//'.txt'
		command_file_name='temp-com3D-'//trim(timestamp())//'.txt'
        tex_file_name='gnu-cairo3D-'//trim(timestamp())//'.tex"'
        stealth=.true.
    end if
 
!open file
    	
	call get_unit(file_unit)	
	if (file_unit==0) then
		print *,'Fatal error in gnuplotter 3D: Could not get a free FORTRAN unit!'
		return
	end if
	open (unit=file_unit, file=data_file_name, status='replace', iostat=ios)	
	if (ios/=0) then
		print *,'Fatal error in gnuplotter 3D: Could not open data file!'
		return
    end if

    nx=DataXYZ%size
    ny=DataXYZ%size2
 
    !create char output arrays
    
    allocate(xyzdat(1:nx, 1:ny)) 

    maxz=0. 
    minz=0. 
 
    select case( DataXYZ % dataType )
            
                case( DATA_SINGLE_PRECISION_TYPE )
 
                    do i=1,nx
                                do j=1,ny
                                           
                                            if(j<ny) then 
                                                write (file_unit,'(e32.16, a)', advance='no' ) DataXYZ%singleValues2(i,j), ', ' 
                                            else 
                                                 write (file_unit,'(e32.16)', advance='yes' ) DataXYZ%singleValues2(i,j) 
                                            end if 
                                            maxz=max(maxz,dble(DataXYZ%singleValues2(i,j)  ))
                                            minz=min(minz,dble(DataXYZ%singleValues2(i,j)  ))
                            
                                end do 
                    end do 
        
                case( DATA_DOUBLE_PRECISION_TYPE )
                    
                    do i=1,nx
                                do j=1,ny
                                            
                                            if(j<ny) then 
                                                write (file_unit,'(e32.16, a)', advance='no' ) DataXYZ%doubleValues2(i,j), ', ' 
                                            else 
                                                 write (file_unit,'(e32.16)', advance='yes' ) DataXYZ%doubleValues2(i,j) 
                                            end if 
                                            maxz=max(maxz,dble(DataXYZ%doubleValues2(i,j)  ))
                                            minz=min(minz,dble(DataXYZ%doubleValues2(i,j)  ))
                            
                                end do 
                    end do 
            
                case( DATA_INTEGER_TYPE )  
                        
                    do i=1,nx
                                do j=1,ny

                                            if(j<ny) then 
                                                write (file_unit,'(i8, a)', advance='no' ) DataXYZ%integerValues2(i,j), ', ' 
                                            else 
                                                 write (file_unit,'(i8)', advance='yes' ) DataXYZ%integerValues2(i,j) 
                                            end if 
                                            
                                            maxz=max(maxz,dble(DataXYZ%integerValues2(i,j)  ))
                                            minz=min(minz,dble(DataXYZ%integerValues2(i,j)  ))
                            
                                end do 
                    end do 
 
                 case default
                 print*, 'Error in gnuplotter: Input data type illegal!'
    end select      

	write (file_unit,'(a)')
 
	close (unit=file_unit) !data file has been prepared
    
    maxz=maxz+self % uppermarginz_
    minz=minz-self % lowermarginz_
 
    !prepare command file
	call get_unit(file_unit)
	if (file_unit==0) then
		print *,'Fatal error in gnuplotter: Could not get a free FORTRAN unit!'
		return
	end if
	open (unit=file_unit, file=command_file_name, status='replace',	iostat=ios)
	if (ios/=0) then
		print *,'Fatal error in gnuplotter: Could not open the terminal command file!'
		return
    end if	
    
    !write gnu commands to command file - gnuplot will read this later 
    
    !read optional setttings
    
    write ( file_unit, '(a)' ) 'reset'
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Data file seperator'
    write ( file_unit, '(a)' ) 'set datafile separator ","'

    if( pdfcairo==.true. .and. terminalme==1 ) then 
            write ( file_unit, '(a)' ) ''
            write ( file_unit, '(a)' ) '# Terminal and output files'
            write ( file_unit, '(a)' ) 'set terminal cairolatex standalone pdf size 16cm,10.5cm dashed transparent \'
            if(color==.false.) write ( file_unit, '(a)' ) ' monochrome header ''\newcommand{\hl}[1]{\setlength{\fboxsep}{0.75pt}\colorbox{white}{#1}}'' '
            if(color==.true.) write ( file_unit, '(a)' ) 'color header ''\newcommand{\hl}[1]{\setlength{\fboxsep}{0.75pt}\colorbox{white}{#1}}'''
            write ( file_unit, '(a)' ) 'set output '//'"'//trim(tex_file_name)//'"'

    elseif( pdfcairo==.false. .and. terminalme==1 ) then
        
        write ( file_unit, '(a)' ) 'set terminal pdf '
        
    else  
            write ( file_unit, '(a)' ) 'set terminal wxt enhanced '
    end if
    
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Grid (keys not active)'
    if (self%grids_==.true.) write ( file_unit, '(a)' ) 'set grid'
                
    write ( file_unit, '(a)' ) 'set title '//'"'//trim(self%title_)//'"'

    if ( abs(self%uppermarginz_)>0.00001d0 .or. abs(self%lowermarginz_)>0.00001d0 ) then
            write ( file_unit, '(a)' ) ''
            write ( file_unit, '(a)' ) '# Axis range'
            write(my_text_tmp1, '(e32.16,a1,e32.16)') minz,':',maxz
		    write ( file_unit, '(a)' ) 'set zrange ['//trim(my_text_tmp1)//']'
    end if 

    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Axis labels'
    write ( file_unit, '(a)' ) 'set xlabel '//'"'//trim(self%xlabel_)//'"'
    write ( file_unit, '(a)' ) 'set ylabel '//'"'//trim(self%ylabel_)//'"'
    write ( file_unit, '(a)' ) 'set zlabel '//'"'//trim(self%zlabel_)//'"'
    write ( file_unit, '(a)' ) 'set nokey'
    
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Color scheme'
	write ( file_unit, '(a)' ) 'set palette model '// trim(default_palette)
    write ( file_unit, '(a)' ) 'set contour base'
     
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# User issued commands'
    if(self%numcommands_>0) then
            do i=1,self%numcommands_
                write ( file_unit, '(a)' ) self%commands_(i)
            end do
    end if 
     
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Labels'
    if(self%numlabels_>0) then
            do i=1,self%numlabels_
                write(my_text_tmp1, '(e32.16)')  self%labels_(i)%xpos 
                write(my_text_tmp2, '(e32.16)')  self%labels_(i)%ypos 
                write(my_text_tmp3, '(e32.16)')  self%labels_(i)%zpos 
                write(my_text_tmp4, '(e32.16)')  self%labels_(i)%rotation
                write ( file_unit, '(a,i2,a)' ) 'set label ', i, ' at  '//trim(my_text_tmp1)//', '//trim(my_text_tmp2)//', '//trim(my_text_tmp3)//' '//'"'//trim(self%labels_(i)%label)//'"'// &
                                                & ' center rotate by '//trim(my_text_tmp3)// ' front' // ' tc rgb ' //'"'// trim(self%labels_(i)%color)//'"'
            end do
    end if 

    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Arrows'
    if(self%numarrows_>0) then
            do i=1,self%numarrows_
                write(my_text_tmp1, '(e32.16)')  self%arrows_(i)%from_xpos
                write(my_text_tmp2, '(e32.16)')  self%arrows_(i)%from_ypos
                write(my_text_tmp3, '(e32.16)')  self%arrows_(i)%from_zpos
                write(my_text_tmp4, '(e32.16)')  self%arrows_(i)%to_xpos
                write(my_text_tmp5, '(e32.16)')  self%arrows_(i)%to_ypos
                write(my_text_tmp5, '(e32.16)')  self%arrows_(i)%to_zpos
               
                
                write ( file_unit, '(a,i2,a)' ) 'set style arrow ', i, ' head filled lw 2'
                write ( file_unit, '(a,i2)' ) 'set arrow arrowstyle ', i
 
                write ( file_unit, '(a,i2,a)' ) 'set arrow ', i, ' from first  '//trim(my_text_tmp1)//', '//trim(my_text_tmp2)//', '//trim(my_text_tmp3) // &
                                                                          ' to '//trim(my_text_tmp4)//', '//trim(my_text_tmp5)//', '//trim(my_text_tmp6) // &
                                                & ' lt 2 linecolor rgb '  //'"'// self%arrows_(i)%color  //'"'// ' lw 2 front size .3, 15 '
            end do
     end if 
 
	!write ( file_unit, '(a)' ) 'set hidden3d'
       
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Plot'
    
	if (pm3d==.false.) write ( file_unit, '(a)' ) 'splot "' // trim ( data_file_name ) //  '" matrix with lines palette title' //'"'//trim(DataXYZ % title)//'"'  
    if(pm3d==.true.) write ( file_unit, '(a)' ) 'splot "' // trim ( data_file_name ) //  '" matrix with pmd3 title' // '"'//trim(DataXYZ % title)//'"' 
    
    deallocate(xyzdat)

    if( pdfcairo==.true. .and. terminalme==1 .and. rungnu==.true. ) then !compile  
            close ( unit = file_unit )
            call run_gnuplot_cairo (command_file_name, tex_file_name, data_file_name, stealth ) 
    else 
            if(rungnu==.true.) then 
                if( terminalme==0 ) then 
                           
                    write ( file_unit, '(a)' ) ''
                    write ( file_unit, '(a)' ) '# Terminate'
                    write ( file_unit, '(a)' ) 'pause -1 "Press a key..."'
                    write ( file_unit, '(a)' ) 'q'
                    close ( unit = file_unit )
                    call run_gnuplot ( 0, command_file_name, output_file_name, data_file_name, stealth )
                else
                    close ( unit = file_unit )
                    call run_gnuplot ( 1, command_file_name, output_file_name, data_file_name, stealth )
                end if 

            else
                close ( unit = file_unit )
            end if 
    end if
    
end subroutine gnuplot3D_auto 

subroutine gnuplot3D_auto_2(self, DataXYZ,  andDataXYZ, terminal, label, stealthin ) 

	implicit none
    integer, parameter		    :: Nc=32, Nclong=128
    class(GnuFigureMaker)       :: self
 
    type(GnuDataSeries)         :: DataXYZ, andDataXYZ 
    
    character(len=*), optional  :: terminal
    character(len=*), optional  :: label
    logical, optional           :: stealthin
    character(len=Nclong)           :: my_text_tmp1, my_text_tmp2, my_text_tmp3, my_text_tmp4, my_text_tmp5, my_text_tmp6
	character(len=Nclong)	    :: output_file_name, data_file_name, data_file_name1, command_file_name, tex_file_name

    character(len=Nc), dimension(:,:), allocatable   :: xyzdat  
    
    character(len=Nc) :: my_line_style 
    
    integer :: arrayn

    real*8 ::   maxz,  minz
    
    integer   :: j, i, ios, file_unit, nmax, nx, ny, terminalme 
    logical   :: stealth 

    if( self%dimension_ .ne. 3 ) then 
        print*, 'Error in gnuplotClass 3D: dimension specification inconsistent!' 
        return
    end if 
    
    if( present(stealthin) ) then 
        stealth=stealthin
    else 
        stealth=stealthmode
    end if 

    !check if output file present 
    if ( present(terminal) ) then 
        if( terminal == 'screen' ) then 
            terminalme=0    
        else 
            terminalme=1
            if( .not. present(label)) then 
           		    print *,'Fatal error in gnuplotter 3D: Must specify output goes to files!'
		            return  
            end if
            
            output_file_name='gnu-fig-'//trim(label)//'.'//trim(terminal)
                        
        end if 
    else
        terminalme=0
    end if
    
 !determine presence of input identifier 
	if (present(label)) then
		data_file_name='gnu-dat3D-'//trim(label)//'-1.txt'
        data_file_name1='gnu-dat3D-'//trim(label)//'-2.txt'
		command_file_name='gnu-com3D-'//trim(label)//'.txt'		
        tex_file_name='gnn-cairo3D-'//trim(label)//'.tex'
	else
		data_file_name='temp-dat3D-'//trim(timestamp())//'-1.txt'
        data_file_name1='temp-dat3D-'//trim(timestamp())//'-2.txt'
		command_file_name='temp-com3D-'//trim(timestamp())//'.txt'
        tex_file_name='gnu-cairo3D-'//trim(timestamp())//'.tex"'
        stealth=.true.
    end if
    
    nx=DataXYZ%size
    ny=DataXYZ%size2
    
    if (andDataXYZ%size2 .ne. ny ) then 
        print*, 'Fatal error in gnuplotter 3D: array size mismatch!'
        return
    end if 
    if (andDataXYZ%size .ne. nx )  then 
        print*,  'Fatal error in gnuplotter 3D: array size mismatch!'
    return
    end if 
    maxz=0.; minz=0. 
    
    !open file --------------------------------------------------------------------------------------------

	call get_unit(file_unit)	
	if (file_unit==0) then
		print *,'Fatal error in gnuplotter 3D: Could not get a free FORTRAN unit!'
		return
	end if
	open (unit=file_unit, file=data_file_name, status='replace', iostat=ios)	
	if (ios/=0) then
		print *,'Fatal error in gnuplotter 3D: Could not open data file!'
		return
    end if
    
    select case( DataXYZ % dataType )
            
                case( DATA_SINGLE_PRECISION_TYPE )
 
                    do i=1,nx
                                do j=1,ny
                                           
                                            if(j<ny) then 
                                                write (file_unit,'(e32.16, a)', advance='no' ) DataXYZ%singleValues2(i,j), ', ' 
                                            else 
                                                 write (file_unit,'(e32.16)', advance='yes' ) DataXYZ%singleValues2(i,j) 
                                            end if 
                                            maxz=max(maxz,dble(DataXYZ%singleValues2(i,j)  ))
                                            minz=min(minz,dble(DataXYZ%singleValues2(i,j)  ))
                            
                                end do 
                    end do 
        
                case( DATA_DOUBLE_PRECISION_TYPE )
                    
                    do i=1,nx
                                do j=1,ny
                                            
                                            if(j<ny) then 
                                                write (file_unit,'(e32.16, a)', advance='no' ) DataXYZ%doubleValues2(i,j), ', ' 
                                            else 
                                                 write (file_unit,'(e32.16)', advance='yes' ) DataXYZ%doubleValues2(i,j) 
                                            end if 
                                            maxz=max(maxz,dble(DataXYZ%doubleValues2(i,j)  ))
                                            minz=min(minz,dble(DataXYZ%doubleValues2(i,j)  ))
                            
                                end do 
                    end do 
            
                case( DATA_INTEGER_TYPE )  
                        
                    do i=1,nx
                                do j=1,ny

                                            if(j<ny) then 
                                                write (file_unit,'(i8, a)', advance='no' ) DataXYZ%integerValues2(i,j), ', ' 
                                            else 
                                                 write (file_unit,'(i8)', advance='yes' ) DataXYZ%integerValues2(i,j) 
                                            end if 
                                            
                                            maxz=max(maxz,dble(DataXYZ%integerValues2(i,j)  ))
                                            minz=min(minz,dble(DataXYZ%integerValues2(i,j)  ))
                            
                                end do 
                    end do 
 
                 case default
                 print*, 'Error in gnuplotter: Input data type illegal!'
                 
    end select   
 
	close (unit=file_unit) 
    
                
    !close file --------------------------------------------------------------------------------------------           
                
    
    !open file --------------------------------------------------------------------------------------------

	call get_unit(file_unit)	
	if (file_unit==0) then
		print *,'Fatal error in gnuplotter 3D: Could not get a free FORTRAN unit!'
		return
	end if
	open (unit=file_unit, file=data_file_name1, status='replace', iostat=ios)	
	if (ios/=0) then
		print *,'Fatal error in gnuplotter 3D: Could not open data file!'
		return
    end if
    
    select case( andDataXYZ % dataType )
            
                case( DATA_SINGLE_PRECISION_TYPE )
 
                    do i=1,nx
                                do j=1,ny
                                           
                                            if(j<ny) then 
                                                write (file_unit,'(e32.16, a)', advance='no' ) andDataXYZ%singleValues2(i,j), ', ' 
                                            else 
                                                 write (file_unit,'(e32.16)', advance='yes' ) andDataXYZ%singleValues2(i,j) 
                                            end if 
                                            maxz=max(maxz,dble(andDataXYZ%singleValues2(i,j)  ))
                                            minz=min(minz,dble(andDataXYZ%singleValues2(i,j)  ))
                            
                                end do 
                    end do 
        
                case( DATA_DOUBLE_PRECISION_TYPE )
                    
                    do i=1,nx
                                do j=1,ny
                                            
                                            if(j<ny) then 
                                                write (file_unit,'(e32.16, a)', advance='no' ) andDataXYZ%doubleValues2(i,j), ', ' 
                                            else 
                                                 write (file_unit,'(e32.16)', advance='yes' ) andDataXYZ%doubleValues2(i,j) 
                                            end if 
                                            maxz=max(maxz,dble(andDataXYZ%doubleValues2(i,j)  ))
                                            minz=min(minz,dble(andDataXYZ%doubleValues2(i,j)  ))
                            
                                end do 
                    end do 
            
                case( DATA_INTEGER_TYPE )  
                        
                    do i=1,nx
                                do j=1,ny

                                            if(j<ny) then 
                                                write (file_unit,'(i8, a)', advance='no' ) andDataXYZ%integerValues2(i,j), ', ' 
                                            else 
                                                 write (file_unit,'(i8)', advance='yes' ) andDataXYZ%integerValues2(i,j) 
                                            end if 
                                            
                                            maxz=max(maxz,dble(andDataXYZ%integerValues2(i,j)  ))
                                            minz=min(minz,dble(andDataXYZ%integerValues2(i,j)  ))
                            
                                end do 
                    end do 
 
                 case default
                 print*, 'Error in gnuplotter: Input data type illegal!'
                 
    end select   
 
	close (unit=file_unit) !data file has been prepared            
                
    !close file --------------------------------------------------------------------------------------------           

    maxz=maxz+self % uppermarginz_
    minz=minz-self % lowermarginz_
 
    !prepare command file
	call get_unit(file_unit)
	if (file_unit==0) then
		print *,'Fatal error in gnuplotter: Could not get a free FORTRAN unit!'
		return
	end if
	open (unit=file_unit, file=command_file_name, status='replace',	iostat=ios)
	if (ios/=0) then
		print *,'Fatal error in gnuplotter: Could not open the terminal command file!'
		return
    end if	
    
    !write gnu commands to command file - gnuplot will read this later 
    
    !read optional setttings
    
    write ( file_unit, '(a)' ) 'reset'
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Datafile seperator'
    write ( file_unit, '(a)' ) 'set datafile separator ","'

    if( pdfcairo==.true. .and. terminalme==1 ) then 
            write ( file_unit, '(a)' ) ''
            write ( file_unit, '(a)' ) '# Terminal and output files'
            write ( file_unit, '(a)' ) 'set terminal cairolatex standalone pdf size 16cm,10.5cm dashed transparent \'
            if(color==.false.) write ( file_unit, '(a)' ) ' monochrome header ''\newcommand{\hl}[1]{\setlength{\fboxsep}{0.75pt}\colorbox{white}{#1}}'' '
            if(color==.true.) write ( file_unit, '(a)' ) 'color header ''\newcommand{\hl}[1]{\setlength{\fboxsep}{0.75pt}\colorbox{white}{#1}}'''
            write ( file_unit, '(a)' ) 'set output '//'"'//trim(tex_file_name)//'"'

    elseif( pdfcairo==.false. .and. terminalme==1 ) then
        
        write ( file_unit, '(a)' ) 'set terminal pdf '
        
    else  
            write ( file_unit, '(a)' ) 'set terminal  wxt enhanced '
    end if
    
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Legend and grids'
    if (self%legend_==.false.) write ( file_unit, '(a)' ) 'set nokey'
    if (self%legend_==.true.) write ( file_unit, '(a)' ) 'set key'
    if (self%grids_==.true.) write ( file_unit, '(a)' ) 'set grid'
                
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Plot title'
    write ( file_unit, '(a)' ) 'set title '//'"'//trim(self%title_)//'"'

    !if ( abs(self%uppermarginz_)>0.00001d0 .or. abs(self%lowermarginz_)>0.00001d0 ) then
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# z range'
    write(my_text_tmp1, '(e32.16,a1,e32.16)') minz,':',maxz
	write ( file_unit, '(a)' ) 'set zrange ['//trim(my_text_tmp1)//']'
    !end if 
          
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Axis labels'
    write ( file_unit, '(a)' ) 'set xlabel '//'"'//trim(self%xlabel_)//'"'
    write ( file_unit, '(a)' ) 'set ylabel '//'"'//trim(self%ylabel_)//'"'
    write ( file_unit, '(a)' ) 'set zlabel '//'"'//trim(self%zlabel_)//'"'
    
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Color mode (inactive when used with keys'
	if (self%legend_==.false.) write ( file_unit, '(a)' ) 'set palette model '// trim(default_palette)
    if (self%legend_==.false.) write ( file_unit, '(a)' ) 'set contour base'
    write ( file_unit, '(a)' ) 'unset colorbox'

    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# User issued commands'
    if(self%numcommands_>0) then
       do i=1,self%numcommands_
             write ( file_unit, '(a)' ) self%commands_(i)
       end do
    end if 
 
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Labels'
    if(self%numlabels_>0) then
            do i=1,self%numlabels_
                write(my_text_tmp1, '(e32.16)')  self%labels_(i)%xpos 
                write(my_text_tmp2, '(e32.16)')  self%labels_(i)%ypos 
                write(my_text_tmp3, '(e32.16)')  self%labels_(i)%zpos 
                write(my_text_tmp4, '(e32.16)')  self%labels_(i)%rotation
                write ( file_unit, '(a,i2,a)' ) 'set label ', i, ' at  '//trim(my_text_tmp1)//', '//trim(my_text_tmp2)//', '//trim(my_text_tmp3)//' '//'"'//trim(self%labels_(i)%label)//'"'// &
                                                & ' center rotate by '//trim(my_text_tmp3)// ' front' // ' tc rgb ' //'"'// trim(self%labels_(i)%color)//'"'
            end do
    end if 

    
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Arrows'
    if(self%numarrows_>0) then
            do i=1,self%numarrows_
                write(my_text_tmp1, '(e32.16)')  self%arrows_(i)%from_xpos
                write(my_text_tmp2, '(e32.16)')  self%arrows_(i)%from_ypos
                write(my_text_tmp3, '(e32.16)')  self%arrows_(i)%from_zpos
                write(my_text_tmp4, '(e32.16)')  self%arrows_(i)%to_xpos
                write(my_text_tmp5, '(e32.16)')  self%arrows_(i)%to_ypos
                write(my_text_tmp5, '(e32.16)')  self%arrows_(i)%to_zpos
               
                write ( file_unit, '(a,i2,a)' ) 'set style arrow ', i, ' head filled lw 2'
                write ( file_unit, '(a,i2)' ) 'set arrow arrowstyle ', i
 
                write ( file_unit, '(a,i2,a)' ) 'set arrow ', i, ' from first  '//trim(my_text_tmp1)//', '//trim(my_text_tmp2)//', '//trim(my_text_tmp3) // &
                                                                          ' to '//trim(my_text_tmp4)//', '//trim(my_text_tmp5)//', '//trim(my_text_tmp6) // &
                                                & ' lt 2 linecolor rgb '  //'"'// self%arrows_(i)%color  //'"'// ' lw 2 front size .3, 15 '
            end do
    end if 
    
	!write ( file_unit, '(a)' ) 'set hidden3d'

    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Plots'
    if (self%legend_==.false.) then 
	    if( pm3d==.false.)  write ( file_unit, '(a)' ) 'splot "' // trim ( data_file_name ) // '" matrix with lines palette title '//'"'//trim(DataXYZ % title )//'", \'
        if( pm3d==.true.)  write ( file_unit, '(a)' ) 'splot "' // trim ( data_file_name ) // '" matrix with pm3d title '//'"'//trim(DataXYZ % title )//'", \'
        write ( file_unit, '(a)' ) '      "' // trim ( data_file_name1 ) // '" matrix with lines palette title '//'"'//trim(andDataXYZ % title )//'"'
    else 
        if( pm3d==.true.)  write ( file_unit, '(a)' ) 'splot "' // trim ( data_file_name ) // '" matrix w pm3d title '//'"'//trim(DataXYZ % title )//'", \'
        if( pm3d==.false.)  write ( file_unit, '(a)' ) 'splot "' // trim ( data_file_name ) // '" matrix with lines title '//'"'//trim(DataXYZ % title )//'", \'
        write ( file_unit, '(a)' ) '      "' // trim ( data_file_name1 ) // '" matrix with lines title '//'"'//trim(andDataXYZ % title )//'"'
    end if 
 
    deallocate(xyzdat)

    if( pdfcairo==.true. .and. terminalme==1 .and. rungnu==.true. ) then !compile  
            close ( unit = file_unit )
            call run_gnuplot_cairo (command_file_name, tex_file_name, data_file_name, stealth, data_file_name1 ) 
    else 
            if(rungnu==.true.) then 
                if( terminalme==0 ) then 
                    
                    write ( file_unit, '(a)' ) ''
                    write ( file_unit, '(a)' ) '# Terminate'
                    write ( file_unit, '(a)' ) 'pause -1 "Press a key..."'
                    write ( file_unit, '(a)' ) 'q'
                    close ( unit = file_unit )
                    call run_gnuplot ( 0, command_file_name, output_file_name, data_file_name, stealth, data_file_name1 )
                else
                    close ( unit = file_unit )
                    call run_gnuplot ( 1, command_file_name, output_file_name, data_file_name, stealth, data_file_name1 )
                end if 

            else
                close ( unit = file_unit )
            end if 
    end if
    
end subroutine gnuplot3D_auto_2
 
function timestamp() result(stamp)
    !creates a string with current date and time 
	    implicit none
            character(len=8)  :: date
            character(len=10) :: time
            character(len=33) :: stamp 
	    call date_and_time(date,time)
	    stamp= 'date_'//date(7:8)//'-'//date(5:6)//'-'//date(1:4)//'_time_'//time(1:2)//'-'//time(3:4)//'-'//time(5:10)
end function timestamp

subroutine gnuhistogram(self, DataX, numbars, terminal, label, stealthin ) 

	implicit none
    integer, parameter		    :: Nc=32, Nclong=128
    class(GnuFigureMaker)       :: self
 
    type(GnuDataSeries)         :: DataX(:)
    integer                     :: numbars
 
    character(len=*), optional  :: terminal
    character(len=*), optional  :: label
    logical, optional           :: stealthin
    
    real*8                       :: xmin, xmax 
    
	character(len=Nclong)	    :: output_file_name, data_file_name, command_file_name, tex_file_name, txt
    
    character(len=Nc)           :: dxtic 
    character(len=Nc)           :: yrange 
    character(len=Nc)           :: xrange1 
    character(len=Nc)           :: xrange2 
    character(len=Nc)           :: xtic_start 
    character(len=Nc)           :: xtic_end 

    real, dimension(:,:), allocatable   :: countoccurences   
    
    
    real :: temp , dx, grid(0:numbars)
    
    integer   ::  n, nx, j, i, item, ios, file_unit, ny, terminalme 
    logical   :: stealth, enter

    !check dimension of data file 
    n=numbars
    
    if( self%dimension_ .ne. 2 ) then 
        print*, 'Error in gnuplotClass 3D: dimension specification inconsistent!' 
        return
    end if 
    
     if( numbars < 2 ) then 
        print*, 'Error in gnuplotClass 3D: two few bars specified!' 
        return
    end if 
    
    if( present(stealthin) ) then 
        stealth=stealthin
    else 
        stealth=stealthmode
    end if 

    !check if output file present 
    
    if ( present(terminal) ) then 
        if( terminal == 'screen' ) then 
            terminalme=0    
        else 
            terminalme=1
            if( .not. present(label)) then 
           		    print *,'Fatal error in gnuplotter 3D: Must specify output goes to files!'
		            return  
            end if
            
            output_file_name='gnu-hist-'//trim(label)//'.'//trim(terminal)
                        
        end if 
    else
        terminalme=0
    end if
    
    !determine presence of input identifier 
    
	if (present(label)) then
		data_file_name='gnu-dat-'//trim(label)//'.txt'
		command_file_name='gnu-com-'//trim(label)//'.txt'		
        tex_file_name='gnn-cairo-'//trim(label)//'.tex'
	else
		data_file_name='temp-dat-'//trim(timestamp())//'.txt'
		command_file_name='temp-com-'//trim(timestamp())//'.txt'
        tex_file_name='gnu-cairo-'//trim(timestamp())//'.tex"'
        stealth=.true.
    end if
 
    !prepare the data
    
    ny= size(DataX)
     
    do j=1,ny
        
     select case ( DataX(j) % dataType )
        
     case( DATA_SINGLE_PRECISION_TYPE )
  
         nx = DataX(j) % size
         if(j==1) xmin=minval(DataX(j) % singleValues(1:nx))
         if(j>1) xmin=min(xmin, dble( minval(DataX(j) % singleValues(1:nx))))
         if(j==1) xmax=maxval(DataX(j) % singleValues(1:nx))
         if(j>1) xmax=max(xmax, dble(maxval(DataX(j) % singleValues(1:nx))))
         
     case( DATA_DOUBLE_PRECISION_TYPE )
         
         nx = DataX(j) % size
         if(j==1) xmin=minval(DataX(j) % doubleValues(1:nx))
         if(j>1) xmin=min(xmin, minval(DataX(j) % doubleValues(1:nx)))
         if(j==1) xmax=maxval(DataX(j) % doubleValues(1:nx))
         if(j>1) xmax=max(xmax, maxval(DataX(j) % doubleValues(1:nx)))
                
     case( DATA_INTEGER_TYPE )  
         
         nx = DataX(j) % size
         if(j==1) xmin= minval(DataX(j) % integerValues(1:nx))
         if(j>1) xmin= min(xmin, dble(minval(DataX(j) % integerValues(1:nx))))
         if(j==1) xmax= maxval(DataX(j) % integerValues(1:nx))
         if(j>1) xmax= max(xmax, dble( maxval(DataX(j) % integerValues(1:nx))))
         
     case default

          print*, 'Error in gnuplotClass: Input data type illegal for the histogram!'
          return 
                 
     end select 
 
    end do 
    
    dx = (xmax-xmin)/real(n)
 
    do i=0,n
		grid(i)=xmin+i*dx
	end do

    allocate(countoccurences(0:n,1:ny))
    countoccurences=0.
    
    do j=1,ny
    
    nx = DataX(j) % size
    
        select case ( DataX(j) % dataType )
        
        case( DATA_SINGLE_PRECISION_TYPE )
    
              do item=1,nx
		           i= min(n, max(0, floor((DataX(j) % singleValues(item)-xmin)/dx)))  
		           countoccurences(i,j)=countoccurences(i,j)+1.
              end do
    
        case( DATA_DOUBLE_PRECISION_TYPE )
        
              do item=1,nx
		           i=min(n, max(0, floor((DataX(j) % doubleValues(item)-xmin)/dx))  )
		           countoccurences(i,j)=countoccurences(i,j)+1.
              end do
                
        case( DATA_INTEGER_TYPE ) 
            
              do item=1,nx
		           i=min(n, max(0, floor((DataX(j) % integerValues(item)-xmin)/dx))  )
		           countoccurences(i,j)=countoccurences(i,j)+1.
              end do
                 
        end select
    
    end do
    
    !Convert to frequency    
    
    do j=1,ny
        temp=sum( countoccurences(0:n,j) )
        do i=0,n
            countoccurences(i,j)=countoccurences(i,j)/(max(temp,temp+0.0000000000001d0))
        end do 
    end do 
  
    !Write data to dat file 

	call get_unit(file_unit)	!get unit and open file
	if (file_unit==0) then 
		print *,'Fatal error in gnuplotter: Could not get a free FORTRAN unit!'
		return
	end if
	open (unit=file_unit, file=data_file_name, status='replace', iostat=ios)	
	if (ios/=0) then
		print *,'Fatal error in gnuplotter: Could not open the terminal data file!'
		return
    end if
 
    !column headings
    if (ny==1) then 
        write (file_unit,'(a)') '# category,      '//trim(DataX(1)%unit)//',       '//trim(DataX(1)%title)
    else 
        write (file_unit,'(a)', advance='no') '# category,      '//trim(DataX(1)%unit)//',      '//trim(DataX(1)%title)
        do j=2,ny-1
            write (file_unit,'(a)', advance='no')  ',       '//  trim(DataX(j)%title)
        end do 
        write (file_unit,'(a)', advance='yes')  ',       '//trim(DataX(j)%title)
    end if
    
    if(ny==1) then 
        do i=0,n
            write (file_unit,'(e32.16, a, e32.16)', advance='yes') grid(i)  , ',    ', countoccurences(i,1) 
        end do
    else 
	    do i=0,n
                write (file_unit,'(e32.16, a, e32.16)', advance='no')  grid(i)  , ',    ', countoccurences(i,1) 
                do j=2,ny-1 
                    write (file_unit,'(a, e32.16)', advance='no') ', ', countoccurences(i,j)
                end do  
                write (file_unit,'(a, e32.16)', advance='yes') ', ', countoccurences(i,j)
        end do
    end if 
            
    close (unit=file_unit) !close dat file
    
   !Write code to com file
 
	call get_unit(file_unit) !get unit and open com file
	if (file_unit==0) then
		print *,'Fatal error in gnuplotter: Could not get a free FORTRAN unit!'
		return
	end if
	open (unit=file_unit, file=command_file_name, status='replace',	iostat=ios)
	if (ios/=0) then
		print *,'Fatal error in gnuplotter: Could not open the terminal command file!'
		return
    end if	
    
    !gnuplot will read this code: 
 
    write ( file_unit, '(a)' ) 'reset'
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Datafile seperator'
    write ( file_unit, '(a)' ) 'set datafile separator ","'
    write ( file_unit, '(a)' ) ''


    if( pdfcairo==.true. .and. terminalme==1 ) then 
            write ( file_unit, '(a)' ) ''
            write ( file_unit, '(a)' ) '# Terminal and output files'
            write ( file_unit, '(a)' ) 'set terminal cairolatex standalone pdf size 16cm,10.5cm dashed transparent \'
            if(color==.false.) write ( file_unit, '(a)' ) ' monochrome header ''\newcommand{\hl}[1]{\setlength{\fboxsep}{0.75pt}\colorbox{white}{#1}}'' '
            if(color==.true.) write ( file_unit, '(a)' ) 'color header ''\newcommand{\hl}[1]{\setlength{\fboxsep}{0.75pt}\colorbox{white}{#1}}'''
            write ( file_unit, '(a)' ) 'set output '//'"'//trim(tex_file_name)//'"'

    elseif( pdfcairo==.false. .and. terminalme==1 ) then
        
        write ( file_unit, '(a)' ) 'set terminal pdf '
        
    else  
        write ( file_unit, '(a)' ) 'set terminal  wxt enhanced '
    end if
 
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Bar width setting'
    write ( file_unit, '(a, e32.16)' ) 'dx=', dx 
    write ( file_unit, '(a, 8i2)' ) 'n=', ny
    write ( file_unit, '(a)' ) 'total_box_width_relative=0.80'
    write ( file_unit, '(a)' ) 'gap_width_relative=0.1'
    write ( file_unit, '(a, i2)' ) 'd_width=(gap_width_relative+total_box_width_relative)*dx/', ny 

 
    if (self%grids_==.true.) then 
        write ( file_unit, '(a)' ) ''
        write ( file_unit, '(a)' ) '# Grid'
        write ( file_unit, '(a)' ) 'set grid'
        write ( file_unit, '(a)' ) 'set style line 12 lc rgb "#808080" lt 0 lw 1'
        write ( file_unit, '(a)' ) 'set grid back ls 12'
    end if 
    
    write ( file_unit, '(a)' ) 'set mxtics 1'
    write ( file_unit, '(a)' ) 'set mytics 1'     
            
    write ( file_unit, '(a)' ) 'set title '//'"'//trim(self%title_)//'"'
            
    write ( file_unit, '(a)' ) 'set xlabel '//'"'//trim(self%xlabel_)//'"'
    write ( file_unit, '(a)' ) 'set ylabel '//'"'//trim(self%ylabel_)//'"'
 
    write ( file_unit, '(a)' ) 'set boxwidth total_box_width_relative/n relative'
    write ( file_unit, '(a)' ) 'set style fill transparent solid 0.5 noborder'
    
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# ox-range'
    write(txt, '(e32.16,a,e32.16)')  grid(0) ,':', grid(n) 
	write ( file_unit, '(a)' ) 'set xrange ['//trim(txt)//']'
 
    write ( file_unit, '(a)' ) ''
    write ( file_unit, '(a)' ) '# Plot'
    if (ny==1) then 
        write ( file_unit, '(a,i2,a)' ) 'plot "' // trim ( data_file_name ) //  '" using ($1+0.6*d_width):', &
                        2, ' with boxes linecolor rgb '//'"'//trim(default_colorA)//'" title ' //'"'//trim(DataX(1) % title)//'"' 
    else 
        write ( file_unit, '(a,i2,a)' ) 'plot "' // trim ( data_file_name ) // '" using ($1+0.6*d_width):',  &
                        2, ' with boxes linecolor rgb ' //'"'//trim(default_colorA)//'" title ' //'"'//trim(DataX(1) % title)//'"'// ', \'
        do j=2, ny-1
            write ( file_unit, '(a,i2,a, i2,a)' ) '     "' // trim ( data_file_name ) //  '" using ($1+0.6*d_width+d_width*', j-1, ':',   &
                        j+1, ' with boxes linecolor rgb '//'"'//trim(colorBank(min(5,j)))//'" title ' //'"'//trim(DataX(j) % title)//'"'// ', \'
        end do 
        write ( file_unit, '(a,i2,a, i2,a)' ) '     "' // trim ( data_file_name ) //'"   using ($1+0.6*d_width+d_width*', j-1, '):',   &
                         ny+1, ' with boxes linecolor rgb '//'"'// trim(colorBank(min(5,ny)))//'" title '//'"'//trim(DataX(ny) % title)//'"' 
    end if 

 
	if( pdfcairo==.true. .and. terminalme==1 .and. rungnu==.true. ) then !compile  
            close ( unit = file_unit )
            call run_gnuplot_cairo (command_file_name, tex_file_name, data_file_name, stealth ) 
    else 
            if(rungnu==.true.) then 
                if( terminalme==0 ) then 
                    
                    write ( file_unit, '(a)' ) ''
                    write ( file_unit, '(a)' ) '# Terminate'
                    write ( file_unit, '(a)' ) 'pause -1 "Press a key..."'
                    write ( file_unit, '(a)' ) 'q'
                    close ( unit = file_unit )
                    call run_gnuplot ( 0, command_file_name, output_file_name, data_file_name, stealth )
                else
                    close ( unit = file_unit )
                    call run_gnuplot ( 1, command_file_name, output_file_name, data_file_name, stealth)
                end if 

            else
                close ( unit = file_unit )
            end if 
    end if
    
 deallocate(countoccurences)
    
 end subroutine gnuhistogram

 subroutine eraseGnuFiles()
 !Issue command to the system to compile

	implicit none

	character (len = 100) :: command
 
	integer :: exitstat, status
    
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
    !*@#$ NOTE
    !             Command to the system that will start GNUPLOT is PLATFORM DEPENDENT.
    !             Must modify to ensure compiler/system compatibility.
    
    write (command, *) 'del gnu-*.txt  '  
	 
	call EXECUTE_COMMAND_LINE(trim(command), .true., exitstat, status)	 	!specit to newer Intel compilers
    
    if (status.ne.0  )  then
        print*, 'Fatal error: Command issued to system failed!'
        return 
    end if 
    if (exitstat.ne.0)  then
        print*, 'Warning: Command issued to the system did not behave as expected!'
        return
    end if 
    
    write (command, *) 'del gnu-*.tex  '  
	 
	call EXECUTE_COMMAND_LINE(trim(command), .true., exitstat, status)	 	!specit to newer Intel compilers
    
    if (status.ne.0  )  then
        print*, 'Fatal error: Command issued to system failed!'
        return 
    end if 
    if (exitstat.ne.0)  then
        print*, 'Warning: Command issued to the system did not behave as expected!'
        return
    end if 
    
    write (command, *) 'del gnu-*.pdf  '  
	 
	call EXECUTE_COMMAND_LINE(trim(command), .true., exitstat, status)	 	!specit to newer Intel compilers
    
    if (status.ne.0  )  then
        print*, 'Fatal error: Command issued to system failed!'
        return 
    end if 
    if (exitstat.ne.0)  then
        print*, 'Warning: Command issued to the system did not behave as expected!'
        return
    end if 
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
     
 end subroutine eraseGnuFiles
 
 subroutine run_gnuplot(mode, command_file, output_file, data_file, stealth, data_file1 )
 !Issue command to the system to compile

	implicit none
    character (len = *), intent(in) :: command_file, data_file 
    character (len = *), intent(in), optional :: data_file1
    integer, intent(in) :: mode
    
    character (len = *), intent(in), optional :: output_file 
    
	character (len = 100) :: command
    
    logical :: stealth  
	
	integer :: exitstat, status
    
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
    !*@#$ NOTE
    !             Command to the system that will start GNUPLOT is PLATFORM DEPENDENT.
    !             Must modify to ensure compiler/system compatibility.
    
    IF( mode == 0) write (command, *) 'gnuplot  ' // trim (command_file) !print on screen
	IF( mode == 1) write (command, *) 'gnuplot < ' // trim (command_file) // ' > ' // trim (output_file) !print to user specified file
    
	call EXECUTE_COMMAND_LINE(trim(command), .true., exitstat, status)	 	!specit to newer Intel compilers
    
    if (status.ne.0  )  then
        print*, 'Fatal error: Command issued to system failed!'
        return 
    end if 
    if (exitstat.ne.0)  then
        print*, 'Warning: Command issued to the system did not behave as expected!'
        return
    end if 
    
    if(stealth==.true.) then
    !clean up
 
    write (command, *) 'del ' // trim (command_file) !delete files
    call EXECUTE_COMMAND_LINE(trim(command), .true., exitstat, status)	 
    if (status.ne.0  )  then
        print*, 'Fatal error: Command issued to system failed!'
        return 
    end if 
    if (exitstat.ne.0)  then
        print*, 'Warning: Command issued to the system did not behave as expected!'
        return
    end if 
    write (command, *) 'del ' // trim (data_file) !delete files
    call EXECUTE_COMMAND_LINE(trim(command), .true., exitstat, status)	 
    if (status.ne.0  )  then
        print*, 'Fatal error: Command issued to system failed!'
        return 
    end if 
    if (exitstat.ne.0)  then
        print*, 'Warning: Command issued to the system did not behave as expected!'
        return
    end if 
    if(present(data_file1)) then 
        write (command, *) 'del ' // trim (data_file1) !delete files
        call EXECUTE_COMMAND_LINE(trim(command), .true., exitstat, status)	 
        if (status.ne.0  )  then
            print*, 'Fatal error: Command issued to system failed!'
            return 
        end if 
        if (exitstat.ne.0)  then
            print*, 'Warning: Command issued to the system did not behave as expected!'
            return
        end if 
        end if 
    end if 
    
	return 
    
end subroutine run_gnuplot

subroutine run_gnuplot_cairo( command_file, tex_file, data_file, stealth, data_file1 )
!Issue command to the system to compile

	implicit none
    character (len = *), intent(in) :: command_file 
    character (len = *), intent(in) :: tex_file, data_file
    character (len = *), intent(in), optional :: data_file1
    
	character (len = 100) :: command
	
	integer :: exitstat, status
    
    logical :: stealth  
     
    !*@#$ NOTE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !             Command to the system that will start GNUPLOT is PLATFORM DEPENDENT.
    !             Must modify to ensure compiler/system compatibility.
    
    !write (command, *) 'path' !print path to check gnuplot is visible in command line
    
    call EXECUTE_COMMAND_LINE(trim(command), .true., exitstat, status)	 
    
    if (status.ne.0  )  then
        print*, 'Fatal error: Command issued to system failed!'
        return 
    end if 
    if (exitstat.ne.0)  then
        print*, 'Warning: Command issued to the system did not behave as expected!'
        return
    end if 
    
	write (command, *) 'gnuplot < ' // trim (command_file) !print to user specified file
    
	call EXECUTE_COMMAND_LINE(trim(command), .true., exitstat, status)	 

    if (status.ne.0  )  then
        print*, 'Fatal error: Command issued to system failed!'
        return 
    end if 
    if (exitstat.ne.0)  then
        print*, 'Warning: Command issued to the system did not behave as expected!'
        return
    end if 
    
    write (command, *) 'pdflatex -synctex=1 -interaction=nonstopmode '//trim(tex_file) 
    
	call EXECUTE_COMMAND_LINE(trim(command), .true., exitstat, status)	  	!specit to newer Intel compilers
 
    if (status.ne.0  )  then
        print*, 'Fatal error: Command issued to system failed!'
        return 
    end if 
 

   !clean up
    if(stealth==.true.) then
 
        write (command, *) 'del ' // trim (command_file) !delete files
        call EXECUTE_COMMAND_LINE(trim(command), .true., exitstat,  status)	 
        if (status.ne.0  )  then
            print*, 'Fatal error: Command issued to system failed!'
            return 
        end if 
        if (exitstat.ne.0)  then
            print*, 'Warning: Command issued to the system did not behave as expected!'
            return
        end if 
        write (command, *) 'del ' // trim (data_file) !delete files
        call EXECUTE_COMMAND_LINE(trim(command), .true., exitstat, status)	 
        if (status.ne.0  )  then
            print*, 'Fatal error: Command issued to system failed!'
            return 
        end if 
        if (exitstat.ne.0)  then
            print*, 'Warning: Command issued to the system did not behave as expected!'
            return
        end if 
        write (command, *) 'del ' // trim (tex_file) !delete files
        call EXECUTE_COMMAND_LINE(trim(command), .true., exitstat, status)	 
        if (status.ne.0  )  then
            print*, 'Fatal error: Command issued to system failed!'
            return 
        end if 
        if (exitstat.ne.0)  then
            print*, 'Warning: Command issued to the system did not behave as expected!'
            return
        end if 
        if(present(data_file1)) then 
            write (command, *) 'del ' // trim (data_file1) !delete files
            call EXECUTE_COMMAND_LINE(trim(command), .true., exitstat, status)	 
            if (status.ne.0  )  then
                print*, 'Fatal error: Command issued to system failed!'
                return 
            end if 
            if (exitstat.ne.0)  then
                print*, 'Warning: Command issued to the system did not behave as expected!'
                return
            end if 
        end if 
    end if 
 
     
   !*@#$ END NOTE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    
	return 
    
end subroutine run_gnuplot_cairo
 
subroutine get_unit(iunit)
 !Return file unit handle
	implicit none
	integer i
	integer ios
	integer iunit
	logical lopen
 
	iunit=0
	do i=1,99
		if (i/= 5 .and. i/=6) then	
			inquire (unit=i, opened=lopen, iostat=ios)
			if (ios==0) then
				if (.not.lopen) then
					iunit=i
					return
				end if
			end if
		
		end if
	end do	
	return
end subroutine get_unit

end module GnuFigureMakerClass 
